package es.jmillandel.senassistant.app;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;

import android.app.Activity;
import android.content.Intent;
import android.graphics.Color;
import android.location.Address;
import android.location.Geocoder;
import android.location.Location;
import android.os.Bundle;
import android.support.v4.app.DialogFragment;
import android.support.v4.app.FragmentActivity;
import android.util.AttributeSet;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.View.OnClickListener;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Toast;

import com.google.android.gms.maps.CameraUpdate;
import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.GoogleMap.OnMapLongClickListener;
import com.google.android.gms.maps.SupportMapFragment;
import com.google.android.gms.maps.GoogleMap.OnMyLocationChangeListener;
import com.google.android.gms.maps.model.BitmapDescriptorFactory;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.Marker;
import com.google.android.gms.maps.model.MarkerOptions;
import com.google.android.gms.maps.model.PolylineOptions;

import es.jmillandel.senassistant.app.R;
import es.jmillandel.senassistant.beans.DialogBoxAlert;
import es.jmillandel.senassistant.beans.LocationUtils;

/**
 * Clase que busca en un mapa por direccion y manteniendo
 * pulsado el marcador devuelve su posicion.
 * 
 * @author Juan Millan
 *
 */
public class BuscarEnMapa extends FragmentActivity {
	public static final String DATO_DEVUELTO1="latitud";
    public static final String DATO_DEVUELTO2="longitud";

	GoogleMap mapa;
	
	private EditText edtNombreDir;
	private Button btnBuscarDir;
	private double latitud;
	private double longitud;
		
	@Override
	public void onCreate(Bundle arg0) {
		super.onCreate(arg0);	
		setContentView(R.layout.buscar_mapa);
		
		edtNombreDir = (EditText) this.findViewById(R.id.edtNombreDir);
		btnBuscarDir = (Button) this.findViewById(R.id.btnBuscarDir);
				
		mapa = ((SupportMapFragment) getSupportFragmentManager().findFragmentById(R.id.map)).getMap();
		
		// asignar vista al mapa
		mapa.setMapType(GoogleMap.MAP_TYPE_NORMAL);
			
	}
	
	@Override
    public void onStart() {
        super.onStart();
        
        btnBuscarDir.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View arg0) {
            	ocultarTeclado();
            	mostrarDireccionMapa();
            }
            
        });
        
        mapa.setOnMapLongClickListener(new OnMapLongClickListener() {
            public void onMapLongClick(LatLng point) {
            	latitud=point.latitude;
            	longitud=point.longitude;
            	devolverDato();
            }
        });
	}
	
	@Override
	public void onDestroy() {
		super.onDestroy();
		
	}
	
	@Override
	public void onResume() {
		super.onResume();
		
	}
	
	/**
	 * muestra el marcador de la direccion en el mapa.
	 * 
	 */
	private void mostrarDireccionMapa() {
		Geocoder gc = new Geocoder(this,Locale.getDefault());
		
		List<Address> addresses;
        try {
            addresses = gc.getFromLocationName(edtNombreDir.getText().toString(),1);
            if(addresses.size() > 0) {
            	mapa.moveCamera(CameraUpdateFactory.newLatLngZoom(new LatLng(addresses.get(0).getLatitude(),addresses.get(0).getLongitude()), 15));
				setMarker(new LatLng(addresses.get(0).getLatitude(),addresses.get(0).getLongitude()),edtNombreDir.getText().toString(),getResources().getString(R.string.buscar_mapa_pulsar),BitmapDescriptorFactory.HUE_RED);
            } else {
            	showDialog(getResources().getString(R.string.error_direccion_noencuentra));
            }

        } catch (IOException e) {
            showDialog(e.getMessage());
        }
		
	}
	
	/**
	 * Aade marcadores al mapa.
	 * 
	 * @param position
	 * @param titulo
	 * @param info
	 * @param bitMap
	 */
	private void setMarker(LatLng position, String titulo, String info, Float bitMap) {
		  Marker myMaker = mapa.addMarker(new MarkerOptions()
		       .position(position)
		       .title(titulo)
		       .snippet(info) 
		       .icon(BitmapDescriptorFactory.defaultMarker(bitMap))); //Color del marcador
	}
	
	/**
	 * muestra una alerta.
	 * 
	 * @param texto
	 */
	private void showDialog(String texto){
		DialogFragment df = new DialogBoxAlert(texto);
	    df.show(this.getSupportFragmentManager(), getResources().getString(R.string.dialogo_titulo));
	}
	
	private void devolverDato() {
        Intent dato = new Intent();
        dato.putExtra(DATO_DEVUELTO1, String.valueOf(latitud));
        dato.putExtra(DATO_DEVUELTO2, String.valueOf(longitud));
        setResult(android.app.Activity.RESULT_OK,dato );
        finish();
    }
	
	private void ocultarTeclado() {
    	InputMethodManager inputManager = (InputMethodManager)getSystemService(this.INPUT_METHOD_SERVICE);
    	inputManager.hideSoftInputFromWindow(edtNombreDir.getWindowToken(), 0);
	}
}
