package es.jmillandel.senassistant.app;

import es.jmillandel.senassistant.app.R;
import es.jmillandel.senassistant.bbdd.GestorSitios;
import es.jmillandel.senassistant.beans.Aviso;
import es.jmillandel.senassistant.beans.Condicion;
import es.jmillandel.senassistant.beans.Destinatario;
import es.jmillandel.senassistant.beans.DialogBoxAlert;
import es.jmillandel.senassistant.beans.DialogBoxConfirm;
import es.jmillandel.senassistant.beans.ExceptionApp;
import es.jmillandel.senassistant.beans.Mensaje;
import es.jmillandel.senassistant.beans.Sitio;
import android.os.Bundle;
import android.app.Activity;
import android.content.Intent;
import android.support.v4.app.DialogFragment;
import android.support.v4.app.FragmentActivity;
import android.util.Log;
import android.view.Menu;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.RadioButton;
import android.widget.TextView;
import android.widget.Toast;

/**
 * Clase que edita los datos de un aviso.
 * 
 * @author Juan Millan
 *
 */
public class EditarAvisos extends FragmentActivity implements DialogBoxConfirm.DialogoBoxConfirmListener {
	public static final int CODIGO_RESPUESTA = 123;
	public static final int CODIGO_RESPUESTA_DESTINA = 456;
	private TextView txtMensaje;
	private TextView txtDestina;
	private Button btnAvisosCancelar;
	private Button btnAvisosGrabar;
	private Button btnAvisosEliminar;
	private ImageButton ibtnAvisosMensajes;
	private ImageButton ibtnAvisosDestinatarios;
	private String esAlta;
	private String clave;
	private GestorSitios gestorSitios;
	private int idSitio;
	private int idMensaje;
	private int idDestinatario;
	private int idCondicion;
	private RadioButton rbtnLlegue;
	private RadioButton rbtnSalga;
	private RadioButton rbtnPase;
	private EditText edtTiempo;
	private boolean alta;
	private boolean valido=true;
	private Sitio sitio=new Sitio();
	private Aviso aviso=new Aviso();
	private Mensaje mensaje=new Mensaje();
	private Destinatario destina=new Destinatario();
	private Condicion condicion;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.editar_avisos);
		
		try{
	        gestorSitios = new GestorSitios(this,GestorSitios.DB_NAME,null,GestorSitios.DB_VERSION);
	        gestorSitios.open();   
        }
        catch (Exception e){
        	showDialog(e.getMessage());
        	return;
        }
		
		Bundle bundle= getIntent().getExtras();
		esAlta=bundle.getString("esAlta");
		clave=bundle.getString("clave");
		idSitio=bundle.getInt("idSitio");

		txtMensaje = (TextView)findViewById(R.id.txtAvisoMensaje);
		txtDestina = (TextView)findViewById(R.id.txtAvisoDestinatario);
		rbtnLlegue = (RadioButton)findViewById(R.id.radioLlegue);
		rbtnSalga = (RadioButton)findViewById(R.id.radioSalga);
		rbtnPase = (RadioButton)findViewById(R.id.radioPase);
		edtTiempo = (EditText)findViewById(R.id.edtAvisosTiempo);
		
		btnAvisosCancelar = (Button) this.findViewById(R.id.btnAvisosCancelar);
		btnAvisosGrabar = (Button) this.findViewById(R.id.btnAvisosGrabar);
		btnAvisosEliminar = (Button) this.findViewById(R.id.btnAvisosEliminar);
		ibtnAvisosMensajes = (ImageButton) this.findViewById(R.id.ibtnAvisosMensajes);
		ibtnAvisosDestinatarios = (ImageButton) this.findViewById(R.id.ibtnAvisosDestinatarios);
		
		if (esAlta.equals("N")) {
			idMensaje=bundle.getInt("idMensaje");
			idDestinatario=bundle.getInt("idDestinatario");
			idCondicion=bundle.getInt("condicion");
			alta=false;
			cargarDatos();
		} else {
			idMensaje=0;
			idDestinatario=0;
			idCondicion=0;
			alta=true;
			edtTiempo.setText("0");
			btnAvisosEliminar.setEnabled(false);
			btnAvisosEliminar.setVisibility(View.INVISIBLE);
		}
	}

	@Override
    public void onStart() {
        super.onStart();
        
        btnAvisosCancelar.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View arg0) {
            	terminar();
            }
            
        });
        
        btnAvisosGrabar.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View arg0) {
            	grabarDatos();
            	if (valido) {
            		terminar();
            	}
            }
            
        });
        
        btnAvisosEliminar.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View arg0) {
            	showConfirm(getResources().getString(R.string.dialogo_confirmar));
            }
            
        });
        
        ibtnAvisosMensajes.setOnClickListener(new OnClickListener() {
 
            @Override
            public void onClick(View arg0) {            	
            	lanzar();
            }
 
        });
        
        ibtnAvisosDestinatarios.setOnClickListener(new OnClickListener() {
 
            @Override
            public void onClick(View arg0) {            	
            	lanzarDest();
            }
 
        });
            
	}
	
	@Override
	public void onDestroy() {
		super.onDestroy();
		
		gestorSitios.close();
	}
	
	private void terminar() {
		gestorSitios.close();
    	finish();
	}
	
	private void cargarDatos() {
		if (!alta && clave.length()==0){
			alta=true;
		}
		
		try {
			if (!alta) {
				aviso=gestorSitios.getAviso(idSitio, idMensaje, idDestinatario, idCondicion);
				
				if (aviso==null) {
					aviso=new Aviso();
					sitio=gestorSitios.getSitio(idSitio);
					mensaje=new Mensaje();
					destina=new Destinatario();
					condicion=Condicion.values()[0];
					alta=true;
					
				} else {
					sitio = gestorSitios.getSitio(idSitio);
					mensaje=gestorSitios.getMensaje(idMensaje);
					destina=gestorSitios.getDestinatario(idDestinatario);
					condicion=aviso.getCondicion();
					txtMensaje.setText(mensaje.getTitulo());
					txtDestina.setText(destina.getNombre());
					rbtnLlegue.setChecked(true);
					rbtnSalga.setChecked(false);
					rbtnPase.setChecked(false);
					switch (aviso.getCondicion().ordinal()) {
						case 0:
							rbtnLlegue.setChecked(true);
							break;
						case 1:
							rbtnSalga.setChecked(true);
							break;
						case 2:
							rbtnPase.setChecked(true);
							break;
					}
					edtTiempo.setText(String.valueOf(aviso.getTiempo()));
				}
			}
		} catch (ExceptionApp e) {
			showDialog(e.getMessage());
		}
	}
	
	private void grabarDatos() {
		if (checkData()) {
			Aviso avisoOld=new Aviso();
			
			try {
				sitio=gestorSitios.getSitio(idSitio);
				
				if (!alta){	
					avisoOld.setSitio(aviso.getSitio());
					avisoOld.setMensaje(aviso.getMensaje());
					avisoOld.setDestinatario(aviso.getDestinatario());
					avisoOld.setCondicion(aviso.getCondicion());
					avisoOld.setTiempo(aviso.getTiempo());
				} else {
					aviso=new Aviso();
				}
				
				aviso.setSitio(sitio);
				mensaje=gestorSitios.getMensaje(txtMensaje.getText().toString());
				aviso.setMensaje(mensaje);
				destina=gestorSitios.getDestinatario(txtDestina.getText().toString());
				aviso.setDestinatario(destina);
				aviso.setCondicion(Condicion.LLEGAR);
				if (rbtnLlegue.isChecked()) {
					aviso.setCondicion(Condicion.LLEGAR);
				}
				if (rbtnSalga.isChecked()) {
					aviso.setCondicion(Condicion.SALIR);
				}
				if (rbtnPase.isChecked()) {
					aviso.setCondicion(Condicion.TIEMPO);
				}
				
				aviso.setTiempo(Long.parseLong(edtTiempo.getText().toString()));
				
				if (alta){
					gestorSitios.altaAviso(aviso);
				} else {
					gestorSitios.modAviso(aviso,avisoOld);
				}
			} catch (ExceptionApp e) {
				showDialog(e.getMessage());
			}
		}
	}
	
	/**
	 * comprueba si los datos son correctos antes de grabar.
	 * 
	 * @return true si son correctos, false en caso contrario
	 */
	private boolean checkData() {
		valido=true;
		
		if (txtMensaje.getText().length()==0) {
			showDialog(getResources().getString(R.string.dialogo_sin_mensaje));
			valido=false;
		}
		
		if (txtDestina.getText().length()==0) {
			showDialog(getResources().getString(R.string.dialogo_sin_destinatario));
			valido=false;
		}
		
		if (rbtnPase.isChecked() && (edtTiempo.getText().length()==0 || Double.parseDouble(edtTiempo.getText().toString())==0.0) ) {
			showDialog(getResources().getString(R.string.dialogo_sin_tiempo));
			valido=false;
		}
				
		return valido;
	}
	
	private void borrarDatos() {
		try {
			gestorSitios.bajaAviso(aviso);
		} catch (ExceptionApp e) {
			showDialog(e.getMessage());
		}
	}
	
	/**
	 * lanza la pantalla para seleccionar un mensaje
	 * 
	 */
	private void lanzar() {
		Intent mensajes = new Intent();
		mensajes.setClass(this,SelecMensajes.class);
    	startActivityForResult( mensajes, CODIGO_RESPUESTA );
		
	}
	
	/**
	 * lanza la pantalla para seleccionar un destinatario
	 * 
	 */
	private void lanzarDest() {
		Intent destinatarios = new Intent();
		destinatarios.setClass(this,SelecDestinatarios.class);
		startActivityForResult( destinatarios, CODIGO_RESPUESTA_DESTINA );
		
	}
	
	protected void onActivityResult(int requestCode,int resultCode, Intent pData) {
        if ( requestCode == CODIGO_RESPUESTA ) {
            if (resultCode == Activity.RESULT_OK ) {
                    final String dato = pData.getExtras().getString(SelecMensajes.DATO_DEVUELTO );
                    txtMensaje.setText(dato);
            }
        }
        
        if ( requestCode == CODIGO_RESPUESTA_DESTINA ) {
	        if (resultCode == Activity.RESULT_OK ) {
	                final String dato = pData.getExtras().getString(SelecMensajes.DATO_DEVUELTO );
	                txtDestina.setText(dato);
	        }
        }
    }
	
	/**
	 * muestra una alerta.
	 * 
	 * @param texto
	 */
	private void showDialog(String texto){
		DialogFragment df = new DialogBoxAlert(texto);
	    df.show(getSupportFragmentManager(), getResources().getString(R.string.dialogo_titulo));
	}
	
	/**
	 * pide confirmacion.
	 * 
	 * @param texto
	 */
	private void showConfirm(String texto){
		DialogFragment df = new DialogBoxConfirm(texto);
	    df.show(getSupportFragmentManager(), getResources().getString(R.string.dialogo_titulo_confirma));
	}
	
	@Override
    public void onDialogPositiveClick(DialogFragment dialog) {
		borrarDatos();
    	terminar();
    }

    @Override
    public void onDialogNegativeClick(DialogFragment dialog) {
    	valido=false;
    }
	
}
