package es.jmillandel.senassistant.app;

import es.jmillandel.senassistant.app.R;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Build;
import android.os.Bundle;
import android.annotation.SuppressLint;
import android.app.Activity;
import android.app.Dialog;
import android.content.Context;
import android.content.IntentSender;
import android.content.SharedPreferences;
//import android.app.AlertDialog;
//import android.app.Dialog;
//import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.Color;
import android.util.Log;
import android.view.Menu;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.RadioButton;
import android.widget.Toast;
import android.support.v4.app.FragmentActivity;
import android.support.v4.app.DialogFragment;
import android.support.v4.app.FragmentTransaction;

import es.jmillandel.senassistant.bbdd.GestorSitios;
import es.jmillandel.senassistant.beans.DialogBoxAlert;
import es.jmillandel.senassistant.beans.ExceptionApp;
import es.jmillandel.senassistant.beans.LocationUtils;
import es.jmillandel.senassistant.beans.RutaSitio;
import es.jmillandel.senassistant.beans.Sitio;

import android.location.Address;
import android.location.Criteria;
import android.location.Geocoder;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;

import java.io.IOException;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import org.w3c.dom.Document;

import com.google.android.gms.common.ConnectionResult;
import com.google.android.gms.common.GooglePlayServicesClient;
import com.google.android.gms.common.GooglePlayServicesUtil;
import com.google.android.gms.location.LocationClient;
import com.google.android.gms.location.LocationRequest;
import com.google.android.gms.maps.CameraUpdate;
import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.SupportMapFragment;
import com.google.android.gms.maps.model.BitmapDescriptorFactory;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.Marker;
import com.google.android.gms.maps.model.MarkerOptions;
import com.google.android.gms.maps.model.PolylineOptions;

/**
 * Clase que muestra el trayecto para ir a un sitio.
 * 
 * @author Juan Millan
 *
 */
public class IrSitio extends FragmentActivity implements LocationListener, GooglePlayServicesClient.ConnectionCallbacks, GooglePlayServicesClient.OnConnectionFailedListener, com.google.android.gms.location.LocationListener {
	GoogleMap mapa;
	
	private String clave;
	private String modo;
	private String modoRuta;
	private GestorSitios gestorSitios;
	private Sitio sitio=new Sitio();
	
	//Gestor del servicio de localizacin
	private LocationRequest mLocationRequest;
	private LocationClient mLocationClient;
    SharedPreferences mPrefs;
    SharedPreferences.Editor mEditor;
    boolean mUpdatesRequested = false;
    private Location locActual;
    private Marker mMarker;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.mapa);
		
		try{
	        gestorSitios = new GestorSitios(this,GestorSitios.DB_NAME,null,GestorSitios.DB_VERSION);
	        gestorSitios.open();   
        }
        catch (Exception e){
        	showDialog(e.getMessage());
        	return;
        }
		
		Bundle bundle= getIntent().getExtras();
		clave=bundle.getString("clave");
		modo=bundle.getString("modo");
		if (modo.equals("Pie")) {
			modoRuta=RutaSitio.MODE_WALKING;
		} else {
			modoRuta=RutaSitio.MODE_DRIVING;
		}
		
		mapa = ((SupportMapFragment) getSupportFragmentManager().findFragmentById(R.id.map)).getMap();
        
		// asignar vista al mapa
		mapa.setMapType(GoogleMap.MAP_TYPE_NORMAL);
		
		//servicio GPS
		// parametros de Location
        mLocationRequest = LocationRequest.create();
        mLocationRequest.setInterval(LocationUtils.UPDATE_INTERVAL_IN_MILLISECONDS);
        mLocationRequest.setPriority(LocationRequest.PRIORITY_HIGH_ACCURACY);
        mLocationRequest.setFastestInterval(LocationUtils.FAST_INTERVAL_CEILING_IN_MILLISECONDS);
        mUpdatesRequested = true;

        // preferencias de la aplicacion
        mPrefs = getSharedPreferences(LocationUtils.SHARED_PREFERENCES, Context.MODE_PRIVATE);

        // editor para preferencias
        mEditor = mPrefs.edit();

        //cliente para Google location service
        mLocationClient = new LocationClient(this, this, this);
	}
		
	@Override
    public void onStop() {
        if (mLocationClient.isConnected()) {
            stopPeriodicUpdates();
        }
        
        mLocationClient.disconnect();

        super.onStop();
    }

	@Override
    public void onStart() {
        super.onStart();
        
        //inicio GPS
        mLocationClient.connect();
        
	}
	
	private void mostrarTrayectoMapa() {
		getLocation();
    	
        sitio=new Sitio();
        try {
	        sitio=gestorSitios.getSitio(clave);
	        if (sitio!=null && locActual!=null) {
	           	mapa.moveCamera(CameraUpdateFactory.newLatLngZoom(new LatLng(locActual.getLatitude(),locActual.getLongitude()), 15));
	        	
	        	LatLng desde = new LatLng(locActual.getLatitude(), locActual.getLongitude());
	            LatLng hasta = new LatLng(sitio.getLatitud(), sitio.getLongitud());
	
	            RutaSitio rs = new RutaSitio();
	
	            Document doc = rs.getRutaSitio(desde, hasta, modoRuta);
	            if (doc!=null) {
		            ArrayList<LatLng> direcciones = rs.getTrayecto(doc);
		            PolylineOptions lineaTrayecto = new PolylineOptions().width(8).color(Color.RED);
		
		            for(int i = 0 ; i < direcciones.size() ; i++) {
		            	if (i==0) {
		            		setMarker(direcciones.get(i),"Inicio","",BitmapDescriptorFactory.HUE_GREEN);
		            	}
		            	lineaTrayecto.add(direcciones.get(i));
		            	if (i==(direcciones.size()-1)) {
		            		setMarker(direcciones.get(i),"Final","",BitmapDescriptorFactory.HUE_RED);
		            	}
		            }
		
		            mapa.addPolyline(lineaTrayecto);
	            } else {
		        	showDialog(getResources().getString(R.string.error_sitio_lee));
		        }
	        } else {
	        	showDialog(getResources().getString(R.string.error_sitio_lee));
	        }
        } catch (ExceptionApp e) {
        	showDialog(e.getMessage());
        }

	}
	
	/**
	 * aade marcadores al mapa.
	 * 
	 * @param position
	 * @param titulo
	 * @param info
	 * @param bitMap
	 */
	private void setMarker(LatLng position, String titulo, String info, Float bitMap) {
		  Marker myMaker = mapa.addMarker(new MarkerOptions()
		       .position(position)
		       .title(titulo)
		       .snippet(info) 
		       .icon(BitmapDescriptorFactory.defaultMarker(bitMap))); //Color del marcador
	}
	
	 @Override
	    public void onPause() {

	        //guarda los parametros actuales para updates
	        mEditor.putBoolean(LocationUtils.KEY_UPDATES_REQUESTED, mUpdatesRequested);
	        mEditor.commit();

	        super.onPause();
	    }
	 
	@Override
    public void onResume() {
        super.onResume();

        // recojo paramero para updates si hay
        if (mPrefs.contains(LocationUtils.KEY_UPDATES_REQUESTED)) {
            mUpdatesRequested = mPrefs.getBoolean(LocationUtils.KEY_UPDATES_REQUESTED, false);

        // detengo updates hasta que se vuelvan a pedir
        } else {
            mEditor.putBoolean(LocationUtils.KEY_UPDATES_REQUESTED, false);
            mEditor.commit();
        }

    }
	
	@Override
	public void onDestroy() {
		super.onDestroy();
		
		gestorSitios.close();
	}
		
	private void showDialog(String texto){
		DialogFragment df = new DialogBoxAlert(texto);
	    df.show(getSupportFragmentManager(), "alerta");
	}
	
     /**
      * metodo para atender los resultados devueltos por  onConnectionFailed 
      * de LocationUpdateRemover y LocationUpdateRequester.
      */
     @Override
     protected void onActivityResult(int requestCode, int resultCode, Intent intent) {

         switch (requestCode) {

             // enviado por onConnectionFailed
             case LocationUtils.CONNECTION_FAILURE_RESOLUTION_REQUEST :

                 switch (resultCode) {
                     // si Google resolvio el problema
                     case Activity.RESULT_OK:
                    	 // nada
                    	 break;

                     // si no
                     default:
                    	 showDialog(getResources().getString(R.string.error_google_contectar));
                    	 break;
                 }

             // otro codigo
             default:
            	 showDialog(getResources().getString(R.string.error_google_servicio));
                break;
         }
     }
     
     /**
      * Comprueba si los servicios de Google estan disponibles.
      *
      * @return true si estan disponibles, false si no
      */
     private boolean servicesConnected() {

         int resultCode = GooglePlayServicesUtil.isGooglePlayServicesAvailable(this);

         if (ConnectionResult.SUCCESS == resultCode) {
             return true;
         } else {
        	 showDialog(getResources().getString(R.string.error_google_servicio));
             return false;
         }
     }
     
     public void getLocation() {
         if (servicesConnected()) {
             Location currentLocation = mLocationClient.getLastLocation();
             locActual=currentLocation;
         }
     }    

     public void startUpdates() {
         mUpdatesRequested = true;
         
          if (servicesConnected()) {
             startPeriodicUpdates();
         }
     }

     public void stopUpdates() {
         mUpdatesRequested = false;

         if (servicesConnected()) {
             stopPeriodicUpdates();
         }
     }
     
     @Override
     public void onConnected(Bundle bundle) {
         if (mUpdatesRequested) {
             startPeriodicUpdates();
         }
         mostrarTrayectoMapa();
     }

     @Override
     public void onDisconnected() {
         
     }

     @Override
     public void onConnectionFailed(ConnectionResult connectionResult) {

         if (connectionResult.hasResolution()) {
             try {
                 //intento solucionar el error
                 connectionResult.startResolutionForResult(this, LocationUtils.CONNECTION_FAILURE_RESOLUTION_REQUEST);
                 
             } catch (IntentSender.SendIntentException e) {
            	 showDialog(e.getMessage());
             }
         } else {

             // si no tiene solucion muestro el error
        	 showDialog(String.valueOf(connectionResult.getErrorCode()));
         }
     }

     @Override
     public void onLocationChanged(Location location) {
    	// borro el marcador anterior
    	if (mMarker!=null) {
 			mMarker.remove();
 		} else {
 			mapa.clear();
 			mostrarTrayectoMapa();
 		}
    	
    	// muestro la posicion en la que me encuentro
 		mMarker = mapa.addMarker(new MarkerOptions()
 		      .position(new LatLng(location.getLatitude(), location.getLongitude()))
 		      .title("Estoy Aqui")
 		      .snippet("") 
 		      .icon(BitmapDescriptorFactory.defaultMarker(BitmapDescriptorFactory.HUE_BLUE)));
 		mapa.animateCamera(CameraUpdateFactory.newLatLng(new LatLng(location.getLatitude(), location.getLongitude())));

     }

     private void startPeriodicUpdates() {
         mLocationClient.requestLocationUpdates(mLocationRequest, this);
     }

     private void stopPeriodicUpdates() {
         mLocationClient.removeLocationUpdates(this);
     }

	@Override
	public void onProviderDisabled(String provider) {
		
		
	}

	@Override
	public void onProviderEnabled(String provider) {
		
	}

	@Override
	public void onStatusChanged(String provider, int status, Bundle extras) {
		
	}

}
