package es.jmillandel.senassistant.app;

import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;

import android.app.Activity;
import android.graphics.Color;
import android.location.Address;
import android.location.Geocoder;
import android.location.Location;
import android.os.Bundle;
import android.support.v4.app.DialogFragment;
import android.support.v4.app.FragmentActivity;
import android.util.AttributeSet;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Toast;

import com.google.android.gms.maps.CameraUpdate;
import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.SupportMapFragment;
import com.google.android.gms.maps.GoogleMap.OnMyLocationChangeListener;
import com.google.android.gms.maps.model.BitmapDescriptorFactory;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.Marker;
import com.google.android.gms.maps.model.MarkerOptions;
import com.google.android.gms.maps.model.PolylineOptions;

import es.jmillandel.senassistant.app.R;
import es.jmillandel.senassistant.bbdd.GestorTrayecto;
import es.jmillandel.senassistant.beans.Aviso;
import es.jmillandel.senassistant.beans.DialogBoxAlert;
import es.jmillandel.senassistant.beans.ExceptionApp;
import es.jmillandel.senassistant.beans.LocationUtils;
import es.jmillandel.senassistant.beans.Sitio;
import es.jmillandel.senassistant.beans.Trayecto;

/**
 * Clase que muestra el trayecto realizado en el dia.
 * 
 * @author Juan Millan
 *
 */
public class MostrarTrayecto extends FragmentActivity {

	GoogleMap mapa;
	
	private GestorTrayecto gestorTrayecto;
	private double latOld;
	private double lonOld;
	private Trayecto traOld;
	private Trayecto trayecto;
	private Location posicion;
	private Location posOld;
	private List<Trayecto> lstTrayecto;

	@Override
	public void onCreate(Bundle arg0) {
		super.onCreate(arg0);
		
		try{
	        gestorTrayecto= new GestorTrayecto(this,GestorTrayecto.DB_NAME,null,GestorTrayecto.DB_VERSION);
	        gestorTrayecto.open();	        
        }
        catch (Exception e){
        	showDialog(e.getMessage());
        	return;
        }
		setContentView(R.layout.mapa);
		
		// inicializo variables
		latOld=0.0;
		lonOld=0.0;
		traOld=null;
		posOld=null;
		trayecto=new Trayecto();
				
		mapa = ((SupportMapFragment) getSupportFragmentManager().findFragmentById(R.id.map)).getMap();
		
		// asignar vista al mapa
		mapa.setMapType(GoogleMap.MAP_TYPE_NORMAL);
		
		mostrarTrayectoMapa();
		
	}
	
	@Override
	public void onDestroy() {
		super.onDestroy();
		
        gestorTrayecto.close();
	}
	
	@Override
	public void onResume() {
		super.onResume();
		
	}
	
	private void mostrarTrayectoMapa()
	{
		try {
			lstTrayecto=gestorTrayecto.listTrayecto();
			boolean inicio=true;
			Trayecto traUlt=new Trayecto();
			
			if (!lstTrayecto.isEmpty()) {
				PolylineOptions lineas = new PolylineOptions();
				Iterator<Trayecto> trayectoIterator = lstTrayecto.iterator();
		        while (trayectoIterator.hasNext()) {
		            Trayecto trayecto = (Trayecto)trayectoIterator.next();
		            if (inicio) {
		            	mapa.moveCamera(CameraUpdateFactory.newLatLngZoom(new LatLng(trayecto.getLatitud(),trayecto.getLongitud()), 15));
						setMarker(new LatLng(trayecto.getLatitud(),trayecto.getLongitud()),"Inicio","",BitmapDescriptorFactory.HUE_GREEN);
						inicio=false;
		        	}
		            lineas.add(new LatLng(trayecto.getLatitud(), trayecto.getLongitud()));
		            traUlt=trayecto;
		        }
		        
		        setMarker(new LatLng(traUlt.getLatitud(),traUlt.getLongitud()),"Final","",BitmapDescriptorFactory.HUE_RED);
		        
		        lineas.width(8);
			    lineas.color(Color.RED);
			    mapa.addPolyline(lineas);
			} else {
				showDialog(getResources().getString(R.string.error_trayecto_noexiste));
			}
		} catch (ExceptionApp e) {
			showDialog(e.getMessage());
		}
	}
	
	/**
	 * aade marcador al mapa.
	 * 
	 * @param position
	 * @param titulo
	 * @param info
	 * @param bitMap
	 */
	private void setMarker(LatLng position, String titulo, String info, Float bitMap) {
		  Marker myMaker = mapa.addMarker(new MarkerOptions()
		       .position(position)
		       .title(titulo)
		       .snippet(info) 
		       .icon(BitmapDescriptorFactory.defaultMarker(bitMap))); //Color del marcador
	}
	
	/**
	 * muestra una alerta.
	 * 
	 * @param texto
	 */
	private void showDialog(String texto){
		DialogFragment df = new DialogBoxAlert(texto);
	    df.show(this.getSupportFragmentManager(), getResources().getString(R.string.dialogo_titulo));
	}
	
}
