package es.jmillandel.senassistant.app;

import java.util.Iterator;
import java.util.List;

import android.os.Bundle;
import android.support.v4.app.DialogFragment;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ListView;
import android.widget.Toast;
import android.app.Activity;
import android.app.ListActivity;
import android.content.Intent;

import es.jmillandel.senassistant.app.R;
import es.jmillandel.senassistant.bbdd.GestorSitios;
import es.jmillandel.senassistant.beans.Aviso;
import es.jmillandel.senassistant.beans.DialogBoxAlert;
import es.jmillandel.senassistant.beans.ExceptionApp;
import es.jmillandel.senassistant.beans.FragmentListActivity;
import es.jmillandel.senassistant.beans.Sitio;

/**
 * Clase que muestra una lista con los avisos de un sitio
 * y permite aadir o seleccionar uno.
 * 
 * @author Juan Millan
 *
 */
public class PantallaAvisos extends FragmentListActivity {
    private String[] avisosNom;
    private List<Aviso> lstAvisos;
    private GestorSitios gestorSitios;
    private Button btnAvisosAnadir;
    private Button btnAvisosSalir;
    private int idSitio;
    private String clave;
    private String esAlta;
    private Aviso clvAviso=new Aviso();

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.avisos);
		
		try{
	        gestorSitios = new GestorSitios(this,GestorSitios.DB_NAME,null,GestorSitios.DB_VERSION);
	        gestorSitios.open();	        
        }
        catch (Exception e){
        	showDialog(e.getMessage());
        	return;
        }
		
		Bundle bundle= getIntent().getExtras();
		idSitio=bundle.getInt("idSitio");
		
		btnAvisosSalir = (Button) this.findViewById(R.id.btnAvisosSalir);
		btnAvisosAnadir = (Button) this.findViewById(R.id.btnAvisosAnadir);
		
		iniPantalla();
	}
	
	@Override
    public void onListItemClick(ListView l, View v, int position, long id) {
        super.onListItemClick(l, v, position, id);
 
        clave=avisosNom[position];
        clvAviso=lstAvisos.get(position);
        esAlta="N";
    	lanzar();
    }
	
	@Override
    public void onStart() {
        super.onStart();
        
        btnAvisosSalir.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View arg0) {
            	gestorSitios.close();
            	finish();
            }
            
        });
        
        btnAvisosAnadir.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View arg0) {
            	esAlta="S";
            	clave="";
            	lanzar();
            }
        });
        
	}

	/**
	 * lanza editar el aviso seleccionado
	 * 
	 */
	private void lanzar() {
		Intent editarAvisos = new Intent();
		if (clave.length()==0 && esAlta.equals("N")) {
			esAlta="S";
		}
		if (esAlta.equals("S")) {
			editarAvisos.putExtra("esAlta", esAlta);
			editarAvisos.putExtra("clave", clave);
			editarAvisos.putExtra("idSitio", idSitio);
		} else {
			editarAvisos.putExtra("esAlta", esAlta);
			editarAvisos.putExtra("clave", clave);
			editarAvisos.putExtra("idSitio", clvAviso.getSitio().getIdSitio());
			editarAvisos.putExtra("idMensaje", clvAviso.getMensaje().getIdMensaje());
			editarAvisos.putExtra("idDestinatario", clvAviso.getDestinatario().getIdDestinatario());
			editarAvisos.putExtra("condicion", clvAviso.getCondicion().ordinal());
		}
		
    	editarAvisos.setClass(this,EditarAvisos.class);
    	startActivity(editarAvisos);
		
	}
	
	@Override
	public void onDestroy() {
		super.onDestroy();
		
		gestorSitios.close();
		
	}
	
	@Override
	public void onResume() {
		super.onResume();
		
		iniPantalla();
		
	}
	
	private void iniPantalla() {
		cargarLista();
		
		// Establecemos el Adapter a la Lista del Fragment
		if (avisosNom!=null){
	        setListAdapter(new ArrayAdapter<String>(this,
	                android.R.layout.simple_list_item_1, avisosNom));
		}
	}
	
	private void cargarLista() {
		try {
			avisosNom=new String[0];
			lstAvisos=gestorSitios.listAvisos(idSitio);
			int i=0;
						
			if (!lstAvisos.isEmpty()) {
				avisosNom=new String[lstAvisos.size()];
				Iterator<Aviso> avisoIterator = lstAvisos.iterator();
		        while (avisoIterator.hasNext()) {
		            Aviso aviso = (Aviso)avisoIterator.next();
		            avisosNom[i]=aviso.getMensaje().getTitulo();
		            if (i==0) {
		            	clave=avisosNom[i];	
		            }
		            i++;
		        }
		        esAlta="N";
			} else {
				esAlta="S";
			}
			
		} catch (ExceptionApp e) {
			showDialog(e.getMessage());
		}
	}
	
	/**
	 * muestra una alerta.
	 * 
	 * @param texto
	 */
	private void showDialog(String texto){
		DialogFragment df = new DialogBoxAlert(texto);
	    df.show(getSupportFragmentManager(), getResources().getString(R.string.dialogo_titulo));
	}
}
