package es.jmillandel.senassistant.app;

import android.content.Intent;
import android.os.Bundle;
import android.support.v4.app.DialogFragment;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentTransaction;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.Toast;

import es.jmillandel.senassistant.app.R;
import es.jmillandel.senassistant.bbdd.GestorGeneral;
import es.jmillandel.senassistant.beans.DatosGenerales;
import es.jmillandel.senassistant.beans.DialogBoxAlert;
import es.jmillandel.senassistant.beans.ExceptionApp;

/**
 * Clase que edita los datos generales y permite
 * el acceso a sitios, mensajes y destinatarios.
 * 
 * @author Juan Millan
 *
 */
public class PantallaConfiguracion extends Fragment {
	private GestorGeneral gestorGeneral;
	private DatosGenerales datGen=new DatosGenerales();
	private EditText edtNombre;
	private EditText edtTelefono;
	private EditText edtContacto;
	private boolean esAlta=false;
	
	public PantallaConfiguracion() {
	}

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
	}

	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
		View rootView = inflater.inflate(R.layout.configuracion, container, false);

		return rootView;
	}
	
	@Override
    public void onStart() {
        super.onStart();
        
        try{
	        gestorGeneral = new GestorGeneral(getActivity(),GestorGeneral.DB_NAME,null,GestorGeneral.DB_VERSION);
	        gestorGeneral.open();
        }
        catch (Exception e){
        	showDialog(e.getMessage());
        	return;
        }
        
        edtNombre = (EditText) getActivity().findViewById(R.id.edtNombre);
        edtTelefono = (EditText) getActivity().findViewById(R.id.edtTelefono);
        edtContacto = (EditText) getActivity().findViewById(R.id.edtContacto);
        
        leerDatosGenerales();
        
        ImageButton ibtnGeneralSitios = (ImageButton) getActivity().findViewById(R.id.ibtnGeneralSitios);
        ibtnGeneralSitios.setOnClickListener(new OnClickListener() {
 
            @Override
            public void onClick(View arg0) {
            	Intent misSitios = new Intent();
            	misSitios.setClass(getActivity(),PantallaSitios.class);
            	startActivity(misSitios);
            }
 
        });
        
        ImageButton ibtnGeneralMensajes = (ImageButton) getActivity().findViewById(R.id.ibtnGeneralMensajes);
        ibtnGeneralMensajes.setOnClickListener(new OnClickListener() {
 
            @Override
            public void onClick(View arg0) {            	
            	Intent mensajes = new Intent();
            	mensajes.setClass(getActivity(),PantallaMensajes.class);
            	startActivity(mensajes);
            }
 
        });
        
        ImageButton ibtnGeneralDestinatarios = (ImageButton) getActivity().findViewById(R.id.ibtnGeneralDestinatarios);
        ibtnGeneralDestinatarios.setOnClickListener(new OnClickListener() {
 
            @Override
            public void onClick(View arg0) {            	
            	Intent destinatarios = new Intent();
        		destinatarios.setClass(getActivity(),PantallaDestinatarios.class);
            	startActivity(destinatarios);
            }
 
        });
    }
	
	@Override
	public void onPause() {
		super.onPause();
		
		// grabar los datos
		grabarDatosGenerales();
		
        gestorGeneral.close();
	}
	
	private void leerDatosGenerales() {
		try {
			datGen = gestorGeneral.getDatosGen();
			
			if (datGen==null) {
				datGen=new DatosGenerales();
				esAlta=true;
				
			} else {
				esAlta=false;
				edtNombre.setText(datGen.getNombre());
				edtTelefono.setText(datGen.getTelefono());
				edtContacto.setText(datGen.getContacto());
			}
		} catch (ExceptionApp e) {
			showDialog(e.getMessage());
		}
	}
	
	private void grabarDatosGenerales() {
		boolean modificado=false;
		
		// compruebo datos
		if (checkData()) {
			if (esAlta) {
				datGen.setNombre(edtNombre.getText().toString());
				datGen.setTelefono(edtTelefono.getText().toString());
				datGen.setContacto(edtContacto.getText().toString());
				try {
					gestorGeneral.altaDatosGen(datGen);
				} catch (ExceptionApp e) {
					showDialog(e.getMessage());
				}
				
			} else {
				String clave=datGen.getNombre();
				// si ha cambiado algo
				if (!datGen.getNombre().equals(edtNombre.getText().toString())) {
					modificado=true;
				}
				if (!datGen.getTelefono().equals(edtTelefono.getText().toString())) {
					modificado=true;
				}
				if (!datGen.getContacto().equals(edtContacto.getText().toString())) {
					modificado=true;
				}
				if (modificado) {
					datGen.setNombre(edtNombre.getText().toString());
					datGen.setTelefono(edtTelefono.getText().toString());
					datGen.setContacto(edtContacto.getText().toString());
					try {
						gestorGeneral.modDatosGen(datGen,clave);
					} catch (ExceptionApp e) {
						showDialog(e.getMessage());
					}
				}
			}
		}
	}
	
	private boolean checkData() {
		// si no hay datos no grabo
		if (edtNombre.getText().length()==0 && edtContacto.getText().length()==0 && edtContacto.getText().length()==0) {
			return false;
		} else {
			return true;
		}
	}
	
	/**
	 * muestra una alerta.
	 * 
	 * @param texto
	 */
	private void showDialog(String texto){
		DialogFragment df = new DialogBoxAlert(texto);
	    df.show(getActivity().getSupportFragmentManager(), getResources().getString(R.string.dialogo_titulo));
	}
	
}
