package es.jmillandel.senassistant.app;

import java.util.Iterator;
import java.util.List;

import android.net.Uri;
import android.os.Bundle;
import android.provider.ContactsContract;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.Toast;
import android.app.Activity;
import android.app.ListActivity;
import android.content.Intent;
import android.database.Cursor;
import android.support.v4.app.DialogFragment;
import es.jmillandel.senassistant.app.R;
import es.jmillandel.senassistant.bbdd.GestorDestinatarios;
import es.jmillandel.senassistant.bbdd.GestorMensajes;
import es.jmillandel.senassistant.beans.Aviso;
import es.jmillandel.senassistant.beans.Destinatario;
import es.jmillandel.senassistant.beans.DialogBoxAlert;
import es.jmillandel.senassistant.beans.DialogBoxConfirm;
import es.jmillandel.senassistant.beans.ExceptionApp;
import es.jmillandel.senassistant.beans.FragmentListActivity;
import es.jmillandel.senassistant.beans.Mensaje;

/**
 * Clase que muestra una lista con los detinatarios
 * y permite editalos.
 * 
 * @author Juan Millan
 *
 */
public class PantallaDestinatarios extends FragmentListActivity implements DialogBoxConfirm.DialogoBoxConfirmListener {
	public static final int CODIGO_RESPUESTA = 123;
	
	private String[] destinaNom;
    private List<Destinatario> lstDestina;
    private GestorDestinatarios gestorDestina;
    private Button btnDestinatariosGrabar;
    private Button btnDestinatariosEliminar;
    private Button btnDestinatariosCancelar;
    private Button btnContactos;
    private EditText edtNombre;
	private EditText edtTelefono;
	private String clave;
    private boolean esAlta;
    private boolean valido=true;
    private Destinatario destina=new Destinatario();
    private boolean swBaja=false;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.destinatarios);
		
		try{
	        gestorDestina = new GestorDestinatarios(this,GestorDestinatarios.DB_NAME,null,GestorDestinatarios.DB_VERSION);
	        gestorDestina.open();	        
        }
        catch (Exception e){
        	showDialog(e.getMessage());
        	return;
        }
		
		btnDestinatariosGrabar = (Button) this.findViewById(R.id.btnDestinatariosGrabar);
		btnDestinatariosEliminar = (Button) this.findViewById(R.id.btnDestinatariosEliminar);
		btnDestinatariosCancelar = (Button) this.findViewById(R.id.btnDestinatariosCancelar);
		btnContactos = (Button) this.findViewById(R.id.btnContactos);
		edtNombre = (EditText) this.findViewById(R.id.edtDestinatariosNombre);
		edtTelefono = (EditText) this.findViewById(R.id.edtDestinatarios_telefono);
		
		iniPantalla();
	}
	
	@Override
    public void onListItemClick(ListView l, View v, int position, long id) {
        super.onListItemClick(l, v, position, id);
 
        clave=destinaNom[position];
        cargarDatos();
    }
	
	@Override
    public void onStart() {
        super.onStart();
        
        btnDestinatariosCancelar.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View arg0) {
            	terminar();
            }
            
        });
        
        btnDestinatariosGrabar.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View arg0) {
            	grabarDatos();
            	iniPantalla();
            }
        });
        
        btnDestinatariosEliminar.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View arg0) {
            	showConfirm(getResources().getString(R.string.dialogo_confirmar));
            } 
        });
        
        // lanza seleccionar datos desde contactos
        btnContactos.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View arg0) {
                Intent intent = new Intent(Intent.ACTION_GET_CONTENT);
            	intent.setType(ContactsContract.CommonDataKinds.Phone.CONTENT_ITEM_TYPE);
            	startActivityForResult(intent, CODIGO_RESPUESTA);
            } 
        });

	}

	private void iniPantalla() {
		cargarLista();
		
		//adapter a la Lista del Fragment
		if (destinaNom!=null){
	        setListAdapter(new ArrayAdapter<String>(this,
	                android.R.layout.simple_list_item_1, destinaNom));
		}
		
		limpiarPantalla();
		btnDestinatariosEliminar.setEnabled(false);
		btnDestinatariosEliminar.setVisibility(View.INVISIBLE);	
	}
	
	private void cargarLista() {
		try {
			destinaNom=new String[0];
			lstDestina=gestorDestina.listDestinatarios();
			int i=0;
			
			if (!lstDestina.isEmpty()) {
				destinaNom=new String[lstDestina.size()];
				Iterator<Destinatario> destinaIterator = lstDestina.iterator();
		        while (destinaIterator.hasNext()) {
		            Destinatario destina = (Destinatario)destinaIterator.next();
		            destinaNom[i]=destina.getNombre();
		            if (i==0) {
		            	clave=destinaNom[i];
		            }
		            i++;
		        }
		        esAlta=false;
			} else {
				esAlta=true;
			}
		} catch (ExceptionApp e) {
			showDialog(e.getMessage());
		}
	}
	
	private void limpiarPantalla() {
		edtNombre.setText("");
		edtTelefono.setText("");
	}
	
	@Override
	public void onDestroy() {
		super.onDestroy();
		
		gestorDestina.close();
	}
	
	private void terminar() {
		gestorDestina.close();
    	finish();
	}
	
	private boolean checkData() {
		valido=true;
		
		if (edtNombre.getText().length()==0) {
			showDialog(getResources().getString(R.string.dialogo_sin_nombre));
			valido=false;
		}
		
		if (edtTelefono.getText().length()==0) {
			showDialog(getResources().getString(R.string.dialogo_sin_telefono));
			valido=false;
		}
		
		return valido;
	}
	
	private void grabarDatos() {
		if (checkData()) {
			// si no existe el destinatario es una alta si no una modificacion
			try {
				destina=gestorDestina.getDestinatario(edtNombre.getText().toString());
				if (destina==null) {
					destina=new Destinatario();
					esAlta=true;
				} else {
					esAlta=false;
				}
				
				destina.setNombre(edtNombre.getText().toString());
				destina.setTelefono(edtTelefono.getText().toString());
				
				if (esAlta){
					try {
						gestorDestina.altaDestinatario(destina);
					} catch (ExceptionApp e) {
						showDialog(e.getMessage());
					}
				} else {
					try {
						gestorDestina.modDestinatario(destina);
					} catch (ExceptionApp e) {
						showDialog(e.getMessage());
					}
				}
				
				clave="";
				btnDestinatariosEliminar.setEnabled(false);
				btnDestinatariosEliminar.setVisibility(View.INVISIBLE);
			} catch (ExceptionApp e) {
				showDialog(e.getMessage());
			}
		}
	}
	
	private void cargarDatos() {
		try {
			destina = gestorDestina.getDestinatario(clave);
			if (destina==null) {
				destina=new Destinatario();
				esAlta=true;
				btnDestinatariosEliminar.setEnabled(false);
				btnDestinatariosEliminar.setVisibility(View.INVISIBLE);
				
			} else {
				edtNombre.setText(destina.getNombre());
				edtTelefono.setText(destina.getTelefono());
				esAlta=false;
				btnDestinatariosEliminar.setEnabled(true);
				btnDestinatariosEliminar.setVisibility(View.VISIBLE);
			}
		} catch (ExceptionApp e) {
			showDialog(e.getMessage());
		}
		
	}
	
	/**
	 * borra un destinatario si no tiene avisos
	 * 
	 */
	private void borrarDatos() {
		Aviso aviso= new Aviso();
		swBaja=false;
		try {
			aviso=gestorDestina.getAvisoDestinatario(destina.getIdDestinatario());
			if (aviso==null) {
				gestorDestina.bajaDestinatario(destina);
				swBaja=true;
			} else {
				showDialog(getResources().getString(R.string.error_destina_conaviso));
			}
		} catch (ExceptionApp e) {
			showDialog(e.getMessage());
		}
	}
	
	/**
	 * muestra una alerta.
	 * 
	 * @param texto
	 */
	private void showDialog(String texto){
		DialogFragment df = new DialogBoxAlert(texto);
	    df.show(getSupportFragmentManager(), getResources().getString(R.string.dialogo_titulo));
	}
	
	/**
	 * pide confirmacion.
	 * 
	 * @param texto
	 */
	private void showConfirm(String texto){
		DialogFragment df = new DialogBoxConfirm(texto);
	    df.show(getSupportFragmentManager(), getResources().getString(R.string.dialogo_titulo_confirma));
	}
	
	@Override
    public void onDialogPositiveClick(DialogFragment dialog) {
		borrarDatos();
		if (swBaja) {
			iniPantalla();
		}
    }

    @Override
    public void onDialogNegativeClick(DialogFragment dialog) {
    	valido=false;
    }
    
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
    	super.onActivityResult(requestCode, resultCode, data);

    	  switch (requestCode) {
    	    case (CODIGO_RESPUESTA) :
    	      if (resultCode == Activity.RESULT_OK) {
    	    	//coge los datos de contactos
    	    	Uri contactData = data.getData();
    	        Cursor c = null;
    	        try {
    	        	
	    	        c =  getContentResolver().query(contactData, new String[]{ 
	                        ContactsContract.CommonDataKinds.Phone.NUMBER,
	                        ContactsContract.CommonDataKinds.Phone.DISPLAY_NAME,
	                        ContactsContract.CommonDataKinds.Phone.TYPE }, null, null, null);
	    	        if (c!= null && c.moveToFirst()) {
	    	          String telefono = c.getString(0);
	    	          String nombre = c.getString(1);
	    	          String tipo = c.getString(2);
	    	          edtNombre.setText(nombre);
	    	  		  edtTelefono.setText(telefono);
	    	        }
    	        } finally {
                    if (c != null) {
                        c.close();
                    }
                }
	    	  }
    	      break;
    	  }
    }
	
}
