package es.jmillandel.senassistant.app;

import java.util.Iterator;
import java.util.List;

import android.os.Bundle;
import android.support.v4.app.DialogFragment;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.Toast;
import android.app.Activity;
import android.app.ListActivity;
import android.content.Intent;
import es.jmillandel.senassistant.app.R;
import es.jmillandel.senassistant.bbdd.GestorMensajes;
import es.jmillandel.senassistant.beans.Aviso;
import es.jmillandel.senassistant.beans.DialogBoxAlert;
import es.jmillandel.senassistant.beans.DialogBoxConfirm;
import es.jmillandel.senassistant.beans.ExceptionApp;
import es.jmillandel.senassistant.beans.FragmentListActivity;
import es.jmillandel.senassistant.beans.Mensaje;

/**
 * Clase que muestra una lista con los mensajes
 * y permite editalos.
 * 
 * @author Juan Millan
 *
 */
public class PantallaMensajes extends FragmentListActivity implements DialogBoxConfirm.DialogoBoxConfirmListener {
    private String[] mensajesTit;
    private List<Mensaje> lstMensajes;
    private GestorMensajes gestorMensajes;
    private Button btnMensajesGrabar;
    private Button btnMensajesEliminar;
    private Button btnMensajesCancelar;
    private EditText edtTitulo;
	private EditText edtTexto;
	private CheckBox chkSitua;
    private String clave;
    private boolean esAlta;
    private boolean valido=true;
    private Mensaje mensaje=new Mensaje();
    private boolean swBaja=false;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.mensajes);
		
		try{
	        gestorMensajes = new GestorMensajes(this,GestorMensajes.DB_NAME,null,GestorMensajes.DB_VERSION);
	        gestorMensajes.open();	        
        }
        catch (Exception e){
        	showDialog(e.getMessage());
        	return;
        }
		
		btnMensajesGrabar = (Button) this.findViewById(R.id.btnMensajesGrabar);
		btnMensajesEliminar = (Button) this.findViewById(R.id.btnMensajesEliminar);
		btnMensajesCancelar = (Button) this.findViewById(R.id.btnMensajesCancelar);
		edtTitulo = (EditText) this.findViewById(R.id.edtMensajesTitulo);
		edtTexto = (EditText) this.findViewById(R.id.edtMensajesTexto);
		chkSitua = (CheckBox) this.findViewById(R.id.chkSituacion);
		
		iniPantalla();
		
	}
	
	@Override
    public void onListItemClick(ListView l, View v, int position, long id) {
        super.onListItemClick(l, v, position, id);
        
        clave=mensajesTit[position];
        cargarDatos();
    }
	
	@Override
    public void onStart() {
        super.onStart();
        
        btnMensajesCancelar.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View arg0) {
            	terminar();
            }
            
        });
        
        btnMensajesGrabar.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View arg0) {
            	grabarDatos();
            	iniPantalla();
            }
        });
        
        btnMensajesEliminar.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View arg0) {
            	showConfirm(getResources().getString(R.string.dialogo_confirmar));
            } 
        });

	}

	private void iniPantalla() {
		cargarLista();
		
		// adapter a la Lista del Fragment
		if (mensajesTit!=null){
	        setListAdapter(new ArrayAdapter<String>(this,
	                android.R.layout.simple_list_item_1, mensajesTit));
		}
		
		limpiarPantalla();
		btnMensajesEliminar.setEnabled(false);
		btnMensajesEliminar.setVisibility(View.INVISIBLE);
		
	}
	
	private void limpiarPantalla() {
		edtTitulo.setText("");
		edtTexto.setText("");
		chkSitua.setChecked(true);
	}
	
	@Override
	public void onDestroy() {
		super.onDestroy();
		
		gestorMensajes.close();
	}
	
	private void terminar() {
		gestorMensajes.close();
    	finish();
	}
	
	private boolean checkData() {
		valido=true;
		
		if (edtTitulo.getText().length()==0) {
			showDialog(getResources().getString(R.string.dialogo_sin_titulo));
			valido=false;
		}
		
		if (edtTexto.getText().length()==0) {
			showDialog(getResources().getString(R.string.dialogo_sin_texto));
			valido=false;
		}
		
		return valido;
	}
	
	private void grabarDatos() {
		if (checkData()) {
			try {
				// si ya no existe el mensaje es una alta si no una modificacion
				mensaje=gestorMensajes.getMensaje(edtTitulo.getText().toString());
				if (mensaje==null) {
					mensaje=new Mensaje();
					esAlta=true;
				} else {
					esAlta=false;
				}
				
				mensaje.setTitulo(edtTitulo.getText().toString());
				mensaje.setTexto(edtTexto.getText().toString());
				mensaje.setSituacion(chkSitua.isChecked());
				
				if (esAlta){
					gestorMensajes.altaMensaje(mensaje);
				} else {
					gestorMensajes.modMensaje(mensaje);
				}
				
				clave="";
				btnMensajesEliminar.setEnabled(false);
				btnMensajesEliminar.setVisibility(View.INVISIBLE);
			} catch (ExceptionApp e) {
				showDialog(e.getMessage());
			}
		}
		
	}
	
	private void cargarDatos() {
		try {
			mensaje = gestorMensajes.getMensaje(clave);
			if (mensaje==null) {
				mensaje=new Mensaje();
				esAlta=true;
				btnMensajesEliminar.setEnabled(false);
				btnMensajesEliminar.setVisibility(View.INVISIBLE);
				
			} else {
				edtTitulo.setText(mensaje.getTitulo());
				edtTexto.setText(mensaje.getTexto());
				chkSitua.setChecked(mensaje.isSituacion());
				esAlta=false;
				btnMensajesEliminar.setEnabled(true);
				btnMensajesEliminar.setVisibility(View.VISIBLE);
			}
		} catch (ExceptionApp e) {
			showDialog(e.getMessage());
		}
	}
	
	private void cargarLista() {
		try {
			mensajesTit=new String[0];
			lstMensajes=gestorMensajes.listMensajes();
			int i=0;
			
			if (!lstMensajes.isEmpty()) {
				mensajesTit=new String[lstMensajes.size()];
				Iterator<Mensaje> mensajeIterator = lstMensajes.iterator();
		        while (mensajeIterator.hasNext()) {
		            Mensaje mensaje = (Mensaje)mensajeIterator.next();
		            mensajesTit[i]=mensaje.getTitulo();
		            if (i==0) {
		            	clave=mensajesTit[i];
		            }
		            i++;
		        }
		        esAlta=false;
			} else {
				esAlta=true;
			}
		} catch (ExceptionApp e) {
			showDialog(e.getMessage());
		}
				
	}
	
	/**
	 * borra los datos de un mensaje si no tiene avisos
	 */
	private void borrarDatos() {
		Aviso aviso= new Aviso();
		swBaja=false;
		try {
			aviso=gestorMensajes.getAvisoMensaje(mensaje.getIdMensaje());
			if (aviso==null) {
				gestorMensajes.bajaMensaje(mensaje);
				swBaja=true;
			} else {
				showDialog(getResources().getString(R.string.error_mensaje_conaviso));
			}
		} catch (ExceptionApp e) {
			showDialog(e.getMessage());
		}
	}
	
	/**
	 * muestra una alerta.
	 * 
	 * @param texto
	 */
	private void showDialog(String texto){
		DialogFragment df = new DialogBoxAlert(texto);
	    df.show(getSupportFragmentManager(), getResources().getString(R.string.dialogo_titulo));
	}
	
	/**
	 * pide confirmacion.
	 * 
	 * @param texto
	 */
	private void showConfirm(String texto){
		DialogFragment df = new DialogBoxConfirm(texto);
	    df.show(getSupportFragmentManager(), getResources().getString(R.string.dialogo_titulo_confirma));
	}
	
	@Override
    public void onDialogPositiveClick(DialogFragment dialog) {
		borrarDatos();
		if (swBaja) {
			iniPantalla();
		}
    }

    @Override
    public void onDialogNegativeClick(DialogFragment dialog) {
    	valido=false;
    }
	
}
