/*
 * Copyright (C) 2010 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package es.jmillandel.senassistant.app;

import android.content.Intent;
import android.location.Location;
import android.os.Bundle;
import android.support.v4.app.FragmentTransaction;
import android.support.v7.app.ActionBar;
import android.support.v7.app.ActionBarActivity;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.webkit.WebView.FindListener;
import android.widget.Toast;

import com.google.android.gms.common.GooglePlayServicesNotAvailableException;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.MapsInitializer;
import com.google.android.gms.maps.SupportMapFragment;

import es.jmillandel.senassistant.app.R;

/**
 * Clase con action bar para el menu principal.
 * 
 * @author Juan Millan
 *
 */
public class PantallaMenu extends ActionBarActivity {
	public GoogleMap mapa;
	private SupportMapFragment mFRaFragment = new PantallaSeguimiento();
	private PantallaTrayecto mFRaTrack = new PantallaTrayecto();
	private PantallaConfiguracion mFRaSettings = new PantallaConfiguracion();
	private FragmentTransaction mTransaction;
	private ActionBar actionBar;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.menu);
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.main, menu);
		
		actionBar=getSupportActionBar();
		actionBar.setHomeButtonEnabled(true);
		actionBar.setDisplayHomeAsUpEnabled(true);
		
		return true;
	}
	
	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch(item.getItemId())
		{
			case R.id.action_trace:
				mTransaction = getSupportFragmentManager().beginTransaction();
				mTransaction.replace(R.id.mainl, mFRaFragment);
				mTransaction.commit();
		
				try {
					MapsInitializer.initialize(this);
				} catch (GooglePlayServicesNotAvailableException e) {
					e.printStackTrace();
				}
				break;
			case R.id.action_track:
				mTransaction = getSupportFragmentManager().beginTransaction();
				mTransaction.replace(R.id.mainl, mFRaTrack);
				mTransaction.commit();
				break;
			case R.id.action_settings:
				mTransaction = getSupportFragmentManager().beginTransaction();
				mTransaction.replace(R.id.mainl, mFRaSettings);
				mTransaction.commit();
				break;
			case android.R.id.home:
				Intent homeIntent = new Intent(this, PantallaMenu.class);
				homeIntent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
				startActivity(homeIntent);
				break;
			default:
				return super.onOptionsItemSelected(item);
		}
		
		return true;
	}
	
 }
