package es.jmillandel.senassistant.app;

import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.concurrent.TimeUnit;

import android.app.Activity;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.IntentSender;
import android.content.SharedPreferences;
import android.graphics.Color;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.location.Address;
import android.location.Criteria;
import android.location.Geocoder;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.BatteryManager;
import android.os.Bundle;
import android.support.v4.app.DialogFragment;
import android.util.AttributeSet;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Toast;

import com.google.android.gms.common.ConnectionResult;
import com.google.android.gms.common.GooglePlayServicesClient;
import com.google.android.gms.common.GooglePlayServicesUtil;
import com.google.android.gms.location.LocationClient;
import com.google.android.gms.location.LocationRequest;
import com.google.android.gms.maps.CameraUpdate;
import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.LocationSource;
import com.google.android.gms.maps.SupportMapFragment;
import com.google.android.gms.maps.GoogleMap.OnMyLocationChangeListener;
import com.google.android.gms.maps.model.BitmapDescriptorFactory;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.Marker;
import com.google.android.gms.maps.model.MarkerOptions;
import com.google.android.gms.maps.model.PolylineOptions;

import es.jmillandel.senassistant.app.R;
import es.jmillandel.senassistant.bbdd.GestorSeguimiento;
import es.jmillandel.senassistant.beans.Aviso;
import es.jmillandel.senassistant.beans.DialogBoxAlert;
import es.jmillandel.senassistant.beans.ExceptionApp;
import es.jmillandel.senassistant.beans.LocationUtils;
import es.jmillandel.senassistant.beans.Sitio;
import es.jmillandel.senassistant.beans.Trayecto;

/**
 * Clase que realiza el seguimiento del usuario.
 * Graba el trayecto, comprueba los sitios y avisos
 * lanzando los mensajes SMS si le corresponde.
 * Tambien lanza mensajes SMS de emergencia si bateria baja
 * o caida. 
 * 
 * @author Juan Millan
 *
 */
public class PantallaSeguimiento extends SupportMapFragment implements SensorEventListener {

	GoogleMap mapa;
	
	private GestorSeguimiento gestorSeguimiento;
	private Sitio sitio;
	private Sitio sitioOld;
	private double latOld;
	private double lonOld;
	private String direccion;
	private Location locationDistance;
	private Aviso avisoLlega;
	private Aviso avisoSale;
	private Aviso avisoPase;
	private boolean swLlega;
	private boolean swSale;
	private boolean swPase;
	private boolean swNuevo;
	private boolean swExit;
	private boolean swFuera;
	private boolean swIni;
	private long tiempo;
	private long tiempoIni;
	private long tiempoFin;
	private List<Sitio> lstSitios;
	private Trayecto traOld;
	private Trayecto trayecto;
	private Location posicion;
	private Location posOld;
	
	private MeLocationSource locationSrc;
	private Marker mMarker;
	
	//bateria baja
	private boolean swBateria=false;
	
	//sensores
	private SensorManager myManager;
    private List<Sensor> sensors;
    private Sensor accSensor;
    private long lastUpdate=0;
    
	@Override
	public void onCreate(Bundle arg0) {
		super.onCreate(arg0);
		
		try{
	        gestorSeguimiento= new GestorSeguimiento(getActivity(),GestorSeguimiento.DB_NAME,null,GestorSeguimiento.DB_VERSION);
	        gestorSeguimiento.open();	        
        }
        catch (Exception e){
        	showDialog(e.getMessage());
        	return;
        }
		
		// inicializo variables
		latOld=0.0;
		lonOld=0.0;
		tiempo=0;
		tiempoIni=0;
		tiempoFin=0;
		swLlega=false;
		swSale=false;
		swPase=false;
		swNuevo=false;
		swExit=false;
		swFuera=true;
		swIni=true;
		direccion="";
		avisoLlega=new Aviso();
		avisoSale=new Aviso();
		avisoPase=new Aviso();
		sitioOld=null;
		traOld=null;
		posOld=null;
		trayecto=new Trayecto();
		lstSitios=new ArrayList<Sitio>();
		try {
			lstSitios=gestorSeguimiento.listSitios();
		} catch (ExceptionApp e) {
			showDialog(e.getMessage());
		}
		
		// GPS
		//Llamar a la clase locationlistener
		locationSrc=new MeLocationSource();
		
		//registro bateria baja
		getActivity().registerReceiver(this.batteryInfoReceiver, new IntentFilter(Intent.ACTION_BATTERY_LOW));
		
		//sensores
		//Sensor + Manager
        myManager = (SensorManager)getActivity().getSystemService(Context.SENSOR_SERVICE);
        sensors = myManager.getSensorList(Sensor.TYPE_ACCELEROMETER);
        if(sensors.size() > 0) {
          accSensor = sensors.get(0);
        }
        myManager.registerListener(this, accSensor, SensorManager.SENSOR_DELAY_NORMAL);
				
	}

	@Override
	public View onCreateView(LayoutInflater mInflater, ViewGroup arg1,
			Bundle arg2) {
		return super.onCreateView(mInflater, arg1, arg2);
	}

	@Override
	public void onInflate(Activity arg0, AttributeSet arg1, Bundle arg2) {
		super.onInflate(arg0, arg1, arg2);
	}

	@Override
	public void onActivityCreated(Bundle savedInstanceState) {
		super.onActivityCreated(savedInstanceState);

		mapa = getMap();
		// asignar vista al mapa
		mapa.setMapType(GoogleMap.MAP_TYPE_NORMAL);
		
		// nivel de zoom
		CameraUpdate zoomCam = CameraUpdateFactory.zoomTo(15);
		mapa.animateCamera(zoomCam);
		
		// cambio location source
        mapa.setLocationSource(locationSrc);

        // zoom por defecto
        mapa.moveCamera(CameraUpdateFactory.zoomTo(15f));
        
	}
	
	@Override
	public void onDestroy() {
		super.onDestroy();
		
		if (locationSrc.mLocationClient.isConnected()) {
            locationSrc.stopPeriodicUpdates();
        }
        
        locationSrc.mLocationClient.disconnect();
		
        gestorSeguimiento.close();
        
        getActivity().unregisterReceiver(batteryInfoReceiver);
        myManager.unregisterListener(this);
	}
	
	@Override
	public void onResume() {
		super.onResume();
		
		try {
			lstSitios=gestorSeguimiento.listSitios();
		} catch (ExceptionApp e) {
			showDialog(e.getMessage());
		}
	}
	
	private void checkLocation(Location location) {
		double lat = location.getLatitude();
		double lon = location.getLongitude();
		
		if (mMarker!=null) {
			mMarker.remove();
		}
		mMarker = mapa.addMarker(new MarkerOptions()
		      .position(new LatLng(location.getLatitude(), location.getLongitude()))
		      .title("")
		      .snippet("") 
		      .icon(BitmapDescriptorFactory.defaultMarker(BitmapDescriptorFactory.HUE_RED))); //Color del marcador
		mapa.animateCamera(CameraUpdateFactory.newLatLng(new LatLng(location.getLatitude(), location.getLongitude())));
		 	
		// si la distancia es > de 5.0 metros (he probado en casa con 5.80 mejor resultado)
		if ((latOld!=lat || lonOld!=lon) && distancia(latOld, lonOld, lat, lon) > LocationUtils.MIN_DISTANCE_TRACE) {
			latOld=lat;
			lonOld=lon;
			posicion=location;
			checkPlace(lat,lon);
		} else {
			// si pasa un tiempo en el sitio y hay aviso 
			if (swPase) {
				checkTime();
			}
		}
	}
	
	private void checkPlace(double lat, double lon) {
		double latSitio=0.0;
		double lonSitio=0.0;
		sitio=new Sitio();
		
		sitio=gestorSeguimiento.getSitioCercano(lat, lon, lstSitios);
		direccion=direccion(lat,lon);
		
		// si sale del sitio y hay aviso
		if ( sitioOld !=null && distancia(sitioOld.getLatitud(), sitioOld.getLongitud(), lat, lon) > LocationUtils.EXIT_DISTANCE_TRACE) {
			swExit=true;
			swFuera=true;
			swPase=false;
		} else {
			swExit=false;
		}
		
		if (swSale && swExit) {
			sendMessageWarning(avisoSale);
			swSale=false;
			swPase=false;
		}
		
		// si pasa un tiempo en el sitio y hay aviso 
		if (swPase) {
			checkTime();
		}
		
		if (sitio!=null) {
			latSitio=sitio.getLatitud();
			lonSitio=sitio.getLongitud();
			
			// si distancia < 10.0 metros las considero iguales
			if (distancia(latSitio, lonSitio, lat, lon) < LocationUtils.MAX_DISTANCE_TRACE) {
				if (sitioOld==null || !sitioOld.getNombre().equals(sitio.getNombre())) {
					swNuevo=true;
				} else {
					swNuevo=false;
				}
				
				// cargo avisos
				if (swNuevo || swFuera) {
					getAvisos(sitio.getIdSitio());
				}
			
				// si llega al sitio y hay aviso
				if (swLlega && (swNuevo || swFuera)) {
					sendMessageWarning(avisoLlega);
					swLlega=false;
				}
				sitioOld=sitio;
				swFuera=false;
			} else {
				swFuera=true;
			}
						
		} else {
			swFuera=true;
		}			
		
		// grabo trayecto
		if (traOld==null || distancia(traOld.getLatitud(),traOld.getLongitud(),lat,lon)>LocationUtils.MAX_DISTANCE_TRACE) {
			if (traOld!=null) {
				PolylineOptions lineas = new PolylineOptions();
				lineas.add(new LatLng(traOld.getLatitud(), traOld.getLongitud()));
				lineas.add(new LatLng(lat, lon));
				lineas.width(8);
			    lineas.color(Color.RED);
			    mapa.addPolyline(lineas);
			}
			grabaTrayecto(lat, lon, posicion);
		}
	}
	
	private void checkTime() {
		tiempoFin= System.currentTimeMillis();
		if ((tiempoFin-tiempoIni)>=tiempo) {
			sendMessageWarning(avisoPase);
			swPase=false;
		}
	}
	
	/**
	 * calcula la distancia entre dos latitudes y longitudes
	 * 
	 * @param lat1
	 * @param lon1
	 * @param lat2
	 * @param lon2
	 * @return dist
	 */
	private double distancia(double lat1, double lon1, double lat2, double lon2) {
		double dist=0;
	    float[] locResult=new float[1];
	    locationDistance.distanceBetween(lat1, lon1, lat2, lon2, locResult);

	    dist=locResult[0];
	    
	    return dist;
	}
	
	/**
	 * busca la direccion a partir de unas coordendas.
	 * 
	 * @param lat
	 * @param lon
	 * @return String con la direccion
	 */
	private String direccion(double lat, double lon) {
		String addressString = "Direccion no encontrada"; 
		
		Geocoder gc = new Geocoder(getActivity(),Locale.getDefault()); 
		try { 
			List<Address> addresses= gc.getFromLocation(lat, lon, 1); 
			StringBuilder sb = new StringBuilder(); 
			if(addresses != null && addresses.size()>0) { 
				Address address = addresses.get(0);
				sb.append(address.getAddressLine(0)).append("\n"); 
				sb.append(address.getLocality()).append("\n"); 
				sb.append(address.getPostalCode()).append("\n"); 
				sb.append(address.getCountryName()); 
			} 
			addressString = sb.toString(); 			
			Toast.makeText(getActivity(), "Direccion: "+addressString, Toast.LENGTH_SHORT).show();
			return addressString;
		} catch (Exception e) {
			return null;
		} 		
	}
	
	/**
	 * obtiene los avisos del sitio localizado.
	 * 
	 * @param idSitio
	 */
	private void getAvisos(int idSitio) {
		List<Aviso> lstAvisos=new ArrayList<Aviso>();
		avisoLlega=new Aviso();
		avisoSale=new Aviso();
		avisoPase=new Aviso();
		swLlega=false;
		swSale=false;
		swPase=false;
		tiempo=0;
		tiempoIni=0;
		tiempoFin=0;
		
		try {
			lstAvisos=gestorSeguimiento.listAvisos(idSitio);
			if (!lstAvisos.isEmpty()) {
				Iterator<Aviso> avisoIterator = lstAvisos.iterator();
		        while (avisoIterator.hasNext()) {
		            Aviso aviso = (Aviso)avisoIterator.next();
		            switch (aviso.getCondicion().ordinal()) {
						case 0:
							swLlega=true;
							avisoLlega=aviso;
							break;
						case 1:
							swSale=true;
							avisoSale=aviso;
							break;
						case 2:
							swPase=true;
							avisoPase=aviso;
							// tiempo en milisegundos
							tiempo=aviso.getTiempo()*60000;
							tiempoIni=System.currentTimeMillis();
							break;
		            }
		        }
			}
		} catch (ExceptionApp e) {
			showDialog(e.getMessage());
		}
	}
	
	/**
	 * envia un mensaje SMS correspondiente a un aviso.
	 * 
	 * @param avisoSMS
	 */
	private void sendMessageWarning(Aviso avisoSMS) {
		String mensajeSMS="";
		
		StringBuilder sb = new StringBuilder();
		sb.append(avisoSMS.getMensaje().getTexto()).append("\n");
		sb.append(direccion).append("\n");
		if (avisoSMS.getMensaje().isSituacion()) {
			sb.append("Latitud: "+latOld).append("\n");
			sb.append("Longitud: "+lonOld).append("\n");
		}
		mensajeSMS=sb.toString();
		try {
			gestorSeguimiento.sendMessage(avisoSMS.getDestinatario().getTelefono(), mensajeSMS);
		} catch (ExceptionApp e) {
			showDialog(e.getMessage());
		}
	}
	
	/**
	 * envia un mensaje SMS si se queda sin bateria o se cae.
	 * 
	 * @param mensaje
	 */
	private void sendMessageSOS(String mensaje) {
		String mensajeSMS="";
		String contacto="";
		
		StringBuilder sb = new StringBuilder();
		sb.append(mensaje).append("\n");
		sb.append(direccion).append("\n");
		sb.append("Latitud: "+latOld).append("\n");
		sb.append("Longitud: "+lonOld).append("\n");
		mensajeSMS=sb.toString();
		try {
			contacto=gestorSeguimiento.getSOSTelefono();
			if (contacto!=null && contacto.length()>0) {
				gestorSeguimiento.sendMessage(contacto, mensajeSMS);
			}
		} catch (ExceptionApp e) {
			showDialog(e.getMessage());
		}
	}
	
	private void grabaTrayecto(double lat, double lon, Location location) {
		trayecto=new Trayecto();
		
		// borro los trayectos pasados si es la primera vez que se ejecuta
		if (swIni) {
			try {
				gestorSeguimiento.bajaTrayectos();
			} catch (ExceptionApp e) {
				showDialog(e.getMessage());
			}
			swIni=false;
		}
		
		trayecto.setDiaHora(new Date());
		trayecto.setDireccion(direccion(lat,lon));
		if (traOld!=null) {
			trayecto.setDistancia(distancia(traOld.getLatitud(),traOld.getLongitud(),lat,lon));
		}
		trayecto.setLatitud(lat);
		trayecto.setLongitud(lon);
		trayecto.setTiempoPerm(location.getTime());
		if (posOld!=null) {
			trayecto.setTiempoTrans(location.getTime() - posOld.getTime());
		}
		
		try {
			gestorSeguimiento.setTrayecto(trayecto);
		} catch (ExceptionApp e) {
			showDialog(e.getMessage());
		}
		
		traOld=trayecto;
		posOld=location;
	}
	
	private void showDialog(String texto){
		DialogFragment df = new DialogBoxAlert(texto);
	    df.show(getActivity().getSupportFragmentManager(), "alerta");
	}
	
	/**
	 * clase que implementa LocationSource para activar
	 * la localizacion en el mapa.
	 *
	 */
	private class MeLocationSource implements LocationSource, LocationListener, GooglePlayServicesClient.ConnectionCallbacks, GooglePlayServicesClient.OnConnectionFailedListener, com.google.android.gms.location.LocationListener {
		private LocationRequest mLocationRequest;
		private LocationClient mLocationClient;
		private OnLocationChangedListener mListener;
		private Context mContext;
	    SharedPreferences mPrefs;
	    SharedPreferences.Editor mEditor;
	    boolean mUpdatesRequested = false;
	    
	    public MeLocationSource() {
	    	// parametros de Location
	        mLocationRequest = LocationRequest.create();
	        mLocationRequest.setInterval(LocationUtils.UPDATE_INTERVAL_IN_MILLISECONDS);
	        mLocationRequest.setPriority(LocationRequest.PRIORITY_HIGH_ACCURACY);
	        mLocationRequest.setFastestInterval(LocationUtils.FAST_INTERVAL_CEILING_IN_MILLISECONDS);
	        mUpdatesRequested = true;

	        // preferencias de la aplicacion
	        mPrefs = getActivity().getSharedPreferences(LocationUtils.SHARED_PREFERENCES, Context.MODE_PRIVATE);

	        // editor para preferencias
	        mEditor = mPrefs.edit();
	        
	        mContext=getActivity().getBaseContext();

	        //cliente para Google location service
	        mLocationClient = new LocationClient(mContext, this, this);
	        
	        //inicio GPS
	        mLocationClient.connect();
	    }

		 /*
	      * metodo para atender los resultados devueltos por  onConnectionFailed 
	      * de LocationUpdateRemover y LocationUpdateRequester.
	      */
	     protected void onActivityResult(int requestCode, int resultCode, Intent intent) {

	         switch (requestCode) {

	             // enviado por onConnectionFailed
	             case LocationUtils.CONNECTION_FAILURE_RESOLUTION_REQUEST :

	                 switch (resultCode) {
	                     // si Google resolvio el problema
	                     case Activity.RESULT_OK:
	                    	 // nada
	                    	 break;

	                     // si no
	                     default:
	                    	 showDialog(getResources().getString(R.string.error_google_contectar));
	                    	 break;
	                 }

	             // otro codigo
	             default:
	            	 showDialog(getResources().getString(R.string.error_google_servicio));
	                break;
	         }
	     }
	     
	     /**
	      * Comprueba si los servicios de Google estan disponibles.
	      *
	      * @return true si estan disponibles, false si no
	      */
	     private boolean servicesConnected() {

	         int resultCode = GooglePlayServicesUtil.isGooglePlayServicesAvailable(mContext);

	         if (ConnectionResult.SUCCESS == resultCode) {
	             return true;
	         } else {
	        	 showDialog(getResources().getString(R.string.error_google_servicio));
	             return false;
	         }
	     }
	     
	     public void getLocation() {
	         if (servicesConnected()) {
	             Location currentLocation = mLocationClient.getLastLocation();
	         }
	     }    

	     public void startUpdates() {
	         mUpdatesRequested = true;

	         if (servicesConnected()) {
	             startPeriodicUpdates();
	         }
	     }

	     public void stopUpdates() {
	         mUpdatesRequested = false;

	         if (servicesConnected()) {
	             stopPeriodicUpdates();
	         }
	     }
	     
	     @Override
	     public void onConnected(Bundle bundle) {
	         if (mUpdatesRequested) {
	             startPeriodicUpdates();
	         }
	     }

	     @Override
	     public void onDisconnected() {
	         
	     }

	     @Override
	     public void onConnectionFailed(ConnectionResult connectionResult) {

	         if (connectionResult.hasResolution()) {
	             try {
	                 //intento solucionar el error
	                 connectionResult.startResolutionForResult(getActivity(), LocationUtils.CONNECTION_FAILURE_RESOLUTION_REQUEST);
	                 
	             } catch (IntentSender.SendIntentException e) {
	            	 showDialog(e.getMessage());
	             }
	         } else {

	             // si no tiene solucion muestro el error
	        	 showDialog(String.valueOf(connectionResult.getErrorCode()));
	         }
	     }

	     @Override
	     public void onLocationChanged(Location location) {
	    	 checkLocation(location);

	     }

	     private void startPeriodicUpdates() {
	         mLocationClient.requestLocationUpdates(mLocationRequest, this);
	     }

	     private void stopPeriodicUpdates() {
	         mLocationClient.removeLocationUpdates(this);
	     }

		@Override
		public void onProviderDisabled(String provider) {
			
			
		}

		@Override
		public void onProviderEnabled(String provider) {
			
		}

		@Override
		public void onStatusChanged(String provider, int status, Bundle extras) {
			
		}

		@Override
		public void activate(OnLocationChangedListener listener) {
			
		}

		@Override
		public void deactivate() {
			
		}
    }
	
	/**
	 * broacastReceiver para bateria baja
	 * 
	 */
	private BroadcastReceiver batteryInfoReceiver = new BroadcastReceiver() {
		@Override
		public void onReceive(Context context, Intent intent) {
			String mensaje=getResources().getString(R.string.sms_sos_bateria).toString();
			
			if (!swBateria) {
				sendMessageSOS(mensaje);
				swBateria=true;
				Toast.makeText(getActivity(), mensaje, Toast.LENGTH_SHORT).show();
			}
		}
	};
	
	// parte del codigo que implementa funciones del acelerometro
	
	@Override
	public void onAccuracyChanged(Sensor sensor, int accuracy) {
		
	}

	@Override
	public void onSensorChanged(SensorEvent event) {
		synchronized (this) {
			if (event.sensor.getType() == Sensor.TYPE_ACCELEROMETER) {
				checkFall( event.values[0], event.values[1], event.values[2], event.timestamp);
			} 
		}
	}
	
	/**
	 * comprueba si hay una caida.
	 * 
	 * @param x
	 * @param y
	 * @param z
	 * @param tiempo
	 */
	 private void checkFall(float x, float y, float z, long tiempo) {
        float actX = x;
        float actY = y;
        float actZ = z;
	     
	    if((tiempo - lastUpdate) >= LocationUtils.CHECK_FALL_IN_MILLISECONDS){
        	double modulo = Math.sqrt(Math.pow(actX,2)+Math.pow(actY,2)+Math.pow(actZ,2));
        	modulo -= SensorManager.GRAVITY_EARTH;
        	if(modulo >= LocationUtils.MAX_ACCELERATION){
        		lastUpdate = tiempo;
        		String mensaje=getResources().getString(R.string.sms_sos_caida).toString();
                sendMessageSOS(mensaje);
                Toast.makeText(getActivity(), mensaje, Toast.LENGTH_SHORT).show();
        	}
        }
	 }
}
