package es.jmillandel.senassistant.app;

import android.os.Bundle;
import android.support.v4.app.DialogFragment;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ListView;
import android.widget.Toast;
import android.app.Activity;
import android.app.ListActivity;
import android.content.Intent;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import es.jmillandel.senassistant.app.R;
import es.jmillandel.senassistant.bbdd.GestorGeneral;
import es.jmillandel.senassistant.bbdd.GestorSitios;
import es.jmillandel.senassistant.beans.DialogBoxAlert;
import es.jmillandel.senassistant.beans.ExceptionApp;
import es.jmillandel.senassistant.beans.FragmentListActivity;
import es.jmillandel.senassistant.beans.Sitio;

/**
 * Clase que muestra una lista con los sitios
 * y permite aadir o seleccionar uno.
 * 
 * @author Juan Millan
 *
 */
public class PantallaSitios extends FragmentListActivity {
	private String[] sitiosNom;
    private List<Sitio> lstSitios;
    private GestorSitios gestorSitios;
    private Button btnSitiosAnadir;
    private Button btnSitiosSalir;
    private String clave;
    private String esAlta;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.mis_sitios);
		
		try{
	        gestorSitios = new GestorSitios(this,GestorSitios.DB_NAME,null,GestorSitios.DB_VERSION);
	        gestorSitios.open();	        
        }
        catch (Exception e){
        	showDialog(e.getMessage());
        	return;
        }
		
		btnSitiosSalir = (Button) this.findViewById(R.id.btnSitiosSalir);
		btnSitiosAnadir = (Button) this.findViewById(R.id.btnSitiosAnadir);
		
		iniPantalla();
		
	}
	
	@Override
    public void onListItemClick(ListView l, View v, int position, long id) {
        super.onListItemClick(l, v, position, id);
 
        clave=sitiosNom[position];
        esAlta="N";
    	lanzar();
    }
	
	@Override
    public void onStart() {
        super.onStart();
        
        btnSitiosSalir.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View arg0) {
            	gestorSitios.close();
            	finish();
            }
            
        });
        
        btnSitiosAnadir.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View arg0) {
            	esAlta="S";
            	clave="";
            	lanzar();
            }
        });
        
	}
	
	@Override
	public void onDestroy() {
		super.onDestroy();
		
		gestorSitios.close();
		
	}
	
	@Override
	public void onResume() {
		super.onResume();
		
		iniPantalla();
		
	}

	private void lanzar() {
		Intent editarSitios = new Intent();
		if (clave.length()==0 && esAlta.equals("N")) {
			esAlta="S";
		}
		editarSitios.putExtra("esAlta", esAlta);
		editarSitios.putExtra("clave", clave);
    	editarSitios.setClass(this,EditarSitios.class);
    	startActivity(editarSitios);
		
	}
	
	private void iniPantalla() {
		cargarLista();
		
		// adapter a la Lista del Fragment
		if (sitiosNom!=null){
	        setListAdapter(new ArrayAdapter<String>(this,
	                android.R.layout.simple_list_item_1, sitiosNom));
		}
	}
	
	private void cargarLista() {
		try {
			sitiosNom=new String[0];
			lstSitios=gestorSitios.listSitios();
			int i=0;
			
			if (!lstSitios.isEmpty()) {
				sitiosNom=new String[lstSitios.size()];
				Iterator<Sitio> sitioIterator = lstSitios.iterator();
		        while (sitioIterator.hasNext()) {
		            Sitio sitio = (Sitio)sitioIterator.next();
		            sitiosNom[i]=sitio.getNombre();
		            if (i==0) {
		            	clave=sitiosNom[i];
		            }
		            i++;
		        }
		        esAlta="N";
			} else {
				esAlta="S";
			}
			
		} catch (ExceptionApp e) {
			showDialog(e.getMessage());
		}
	}
	
	/**
	 * muestra una alerta.
	 * 
	 * @param texto
	 */
	private void showDialog(String texto){
		DialogFragment df = new DialogBoxAlert(texto);
	    df.show(getSupportFragmentManager(), getResources().getString(R.string.dialogo_titulo));
	}
	
}
