package es.jmillandel.senassistant.app;

import android.app.Activity;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.support.v4.app.DialogFragment;
import android.support.v4.app.Fragment;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.View.OnClickListener;
import android.widget.ImageButton;

import es.jmillandel.senassistant.app.R;
import es.jmillandel.senassistant.bbdd.GestorSitios;
import es.jmillandel.senassistant.beans.DialogBoxAlert;
import es.jmillandel.senassistant.beans.Sitio;

/**
 * Clase que permite seleccionar ir a un sitio
 * o mostrar el trayecto realizado hasta el momento.
 * 
 * @author Juan Millan
 *
 */
public class PantallaTrayecto extends Fragment {
	public static final int CODIGO_RESPUESTA = 789;
	
	private GestorSitios gestorSitios;
	private ImageButton ibtnMostrarTrayecto;
	private ImageButton ibtnTrayectoIrSitio;
	private String nomSitio;
	private String modo;
	private Sitio sitio;

	public PantallaTrayecto() {
	}

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		try{
	        gestorSitios = new GestorSitios(getActivity(),GestorSitios.DB_NAME,null,GestorSitios.DB_VERSION);
	        gestorSitios.open();   
        }
        catch (Exception e){
        	showDialog(e.getMessage());
        	return;
        }
		
	}

	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
		View rootView = inflater.inflate(R.layout.trayecto, container, false);

		return rootView;
	}
	
	@Override
    public void onStart() {
        super.onStart();
     
        ibtnMostrarTrayecto = (ImageButton) getActivity().findViewById(R.id.ibtnMostrarTrayecto);
        ibtnTrayectoIrSitio = (ImageButton) getActivity().findViewById(R.id.ibtnTrayectoIrSitio);
        
        ibtnMostrarTrayecto.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View arg0) {            	
            	lanzar();
            }
        });
        
        ibtnTrayectoIrSitio.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View arg0) {            	
            	lanzarIrSitio();
            }
        });
	}
	
	@Override
	public void onDestroy() {
		super.onDestroy();
		
		gestorSitios.close();
	}
	
	/**
	 * lanza mostrar trayecto realizado hasta el momento
	 * 
	 */
	private void lanzar() {
		Intent mostrar = new Intent();
		mostrar.setClass(getActivity(),MostrarTrayecto.class);
    	startActivity(mostrar);
		
	}
	
	/**
	 * lanza ir a un sitio desde la posicion
	 * en la que se encuentra el usuario
	 * 
	 */
	private void lanzarIrSitio() {
		Intent selecSitio = new Intent();
		selecSitio.setClass(getActivity(),SelecSitios.class);
    	startActivityForResult( selecSitio, CODIGO_RESPUESTA );	
	}
	
	public void onActivityResult(int requestCode,int resultCode, Intent pData) {
        if ( requestCode == CODIGO_RESPUESTA ) {
            if (resultCode == Activity.RESULT_OK ) {
                    String dato = pData.getExtras().getString(SelecSitios.DATO_DEVUELTO1 );
                    nomSitio=dato;
                    dato = pData.getExtras().getString(SelecSitios.DATO_DEVUELTO2 );
                    modo=dato;
                    Intent irSitio = new Intent();
            		irSitio.setClass(getActivity(),IrSitio.class);
            		irSitio.putExtra("clave", nomSitio);
            		irSitio.putExtra("modo", modo);
                	startActivity(irSitio);
            }
        }
    }
	
	/**
	 * muestra una alerta.
	 * 
	 * @param texto
	 */
	private void showDialog(String texto){
		DialogFragment df = new DialogBoxAlert(texto);
	    df.show(getActivity().getSupportFragmentManager(), getResources().getString(R.string.dialogo_titulo));
	}
}
