package es.jmillandel.senassistant.app;

import java.util.Iterator;
import java.util.List;

import android.os.Bundle;
import android.support.v4.app.DialogFragment;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.Toast;
import android.app.Activity;
import android.app.ListActivity;
import android.content.Intent;
import es.jmillandel.senassistant.app.R;
import es.jmillandel.senassistant.bbdd.GestorDestinatarios;
import es.jmillandel.senassistant.beans.Destinatario;
import es.jmillandel.senassistant.beans.DialogBoxAlert;
import es.jmillandel.senassistant.beans.DialogBoxConfirm;
import es.jmillandel.senassistant.beans.ExceptionApp;
import es.jmillandel.senassistant.beans.FragmentListActivity;

/**
 * Clase que muestra una lista con los destinatarios
 * y permite seleccionar uno.
 * 
 * @author Juan Millan
 *
 */
public class SelecDestinatarios extends FragmentListActivity {
    private String[] destinaNom;
    private List<Destinatario> lstDestina;
    private GestorDestinatarios gestorDestinatarios;
    private Button btnDestinatariosCancelar;
    private String clave;
    private boolean esAlta;
    private boolean valido=true;
    private Destinatario destina=new Destinatario();
    public static final String DATO_DEVUELTO="";

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.selec_destinatario);
		
		try{
	        gestorDestinatarios = new GestorDestinatarios(this,GestorDestinatarios.DB_NAME,null,GestorDestinatarios.DB_VERSION);
	        gestorDestinatarios.open();
	        
        }
        catch (Exception e){
        	showDialog(e.getMessage());
        	return;
        }
		
		btnDestinatariosCancelar = (Button) this.findViewById(R.id.btnDestinatariosCancelar);
		
		iniPantalla();
		
	}
	
	@Override
    public void onListItemClick(ListView l, View v, int position, long id) {
        super.onListItemClick(l, v, position, id);
        
        clave=destinaNom[position];
        devolverDato();
    }
	
	@Override
    public void onStart() {
        super.onStart();
        
        btnDestinatariosCancelar.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View arg0) {
            	terminar();
            }
            
        });

	}

	private void iniPantalla() {
		cargarLista();
		
		// adapter a la Lista del Fragment
		if (destinaNom!=null){
	        setListAdapter(new ArrayAdapter<String>(this,
	                android.R.layout.simple_list_item_1, destinaNom));
		}
		
	}
	
	private void terminar() {
		gestorDestinatarios.close();
    	finish();
	}
		
	private void cargarLista() {
		try {
			destinaNom=new String[0];
			lstDestina=gestorDestinatarios.listDestinatarios();
			int i=0;
			
			if (!lstDestina.isEmpty()) {
				destinaNom=new String[lstDestina.size()];
				Iterator<Destinatario> destinaIterator = lstDestina.iterator();
		        while (destinaIterator.hasNext()) {
		            Destinatario destina = (Destinatario)destinaIterator.next();
		            destinaNom[i]=destina.getNombre();
		            if (i==0) {
		            	clave=destinaNom[i];
		            }
		            i++;
		        }
		        esAlta=false;
			} else {
				esAlta=true;
			}
		} catch (ExceptionApp e) {
			showDialog(e.getMessage());
		}
				
	}
	
	private void devolverDato()
    {
        Intent dato = new Intent();
        dato.putExtra(DATO_DEVUELTO, clave);
        setResult(android.app.Activity.RESULT_OK,dato );
        terminar();
    }
	
	/**
	 * muestra una alerta.
	 * 
	 * @param texto
	 */
	private void showDialog(String texto){
		DialogFragment df = new DialogBoxAlert(texto);
	    df.show(getSupportFragmentManager(), getResources().getString(R.string.dialogo_titulo));
	}
	
}
