package es.jmillandel.senassistant.app;

import java.util.Iterator;
import java.util.List;

import android.os.Bundle;
import android.support.v4.app.DialogFragment;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.Toast;
import android.app.Activity;
import android.app.ListActivity;
import android.content.Intent;
import es.jmillandel.senassistant.app.R;
import es.jmillandel.senassistant.bbdd.GestorMensajes;
import es.jmillandel.senassistant.beans.DialogBoxAlert;
import es.jmillandel.senassistant.beans.DialogBoxConfirm;
import es.jmillandel.senassistant.beans.ExceptionApp;
import es.jmillandel.senassistant.beans.FragmentListActivity;
import es.jmillandel.senassistant.beans.Mensaje;

/**
 * Clase que muestra una lista con los mensajes
 * y permite seleccionar uno.
 * 
 * @author Juan Millan
 *
 */
public class SelecMensajes extends FragmentListActivity {
    private String[] mensajesTit;
    private List<Mensaje> lstMensajes;
    private GestorMensajes gestorMensajes;
    private Button btnMensajesCancelar;
    private String clave;
    private boolean esAlta;
    private boolean valido=true;
    private Mensaje mensaje=new Mensaje();
    public static final String DATO_DEVUELTO="";

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.selec_mensaje);
		
		try{
	        gestorMensajes = new GestorMensajes(this,GestorMensajes.DB_NAME,null,GestorMensajes.DB_VERSION);
	        gestorMensajes.open();
	        
        }
        catch (Exception e){
        	showDialog(e.getMessage());
        	return;
        }
		
		btnMensajesCancelar = (Button) this.findViewById(R.id.btnMensajesCancelar);
		
		iniPantalla();
		
	}
	
	@Override
    public void onListItemClick(ListView l, View v, int position, long id) {
        super.onListItemClick(l, v, position, id);
        
        clave=mensajesTit[position];
        devolverDato();
    }
	
	@Override
    public void onStart() {
        super.onStart();
        
        btnMensajesCancelar.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View arg0) {
            	terminar();
            }
            
        });

	}

	private void iniPantalla() {
		cargarLista();
		
		// adapter a la Lista del Fragment
		if (mensajesTit!=null){
	        setListAdapter(new ArrayAdapter<String>(this,
	                android.R.layout.simple_list_item_1, mensajesTit));
		}
		
	}
	
	private void terminar() {
		gestorMensajes.close();
    	finish();
	}
		
	private void cargarLista() {
		try {
			mensajesTit=new String[0];
			lstMensajes=gestorMensajes.listMensajes();
			
			int i=0;
			
			if (!lstMensajes.isEmpty()) {
				mensajesTit=new String[lstMensajes.size()];
				Iterator<Mensaje> mensajeIterator = lstMensajes.iterator();
		        while (mensajeIterator.hasNext()) {
		            Mensaje mensaje = (Mensaje)mensajeIterator.next();
		            mensajesTit[i]=mensaje.getTitulo();
		            if (i==0) {
		            	clave=mensajesTit[i];
		            }
		            i++;
		        }
		        esAlta=false;
			} else {
				esAlta=true;
			}
		} catch (ExceptionApp e) {
			showDialog(e.getMessage());
		}
	}
	
	private void devolverDato()
    {
        Intent dato = new Intent();
        dato.putExtra(DATO_DEVUELTO, clave);
        setResult(android.app.Activity.RESULT_OK,dato );
        terminar();
    }
	
	/**
	 * muestra una alerta.
	 * 
	 * @param texto
	 */
	private void showDialog(String texto){
		DialogFragment df = new DialogBoxAlert(texto);
	    df.show(getSupportFragmentManager(), getResources().getString(R.string.dialogo_titulo));
	}
}
