package es.jmillandel.senassistant.app;

import java.util.Iterator;
import java.util.List;

import android.os.Bundle;
import android.support.v4.app.DialogFragment;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.RadioButton;
import android.widget.Toast;
import android.app.Activity;
import android.app.ListActivity;
import android.content.Intent;
import es.jmillandel.senassistant.app.R;
import es.jmillandel.senassistant.bbdd.GestorSitios;
import es.jmillandel.senassistant.beans.DialogBoxAlert;
import es.jmillandel.senassistant.beans.DialogBoxConfirm;
import es.jmillandel.senassistant.beans.ExceptionApp;
import es.jmillandel.senassistant.beans.FragmentListActivity;
import es.jmillandel.senassistant.beans.Sitio;

/**
 * Clase que muestra una lista con los sitios
 * y permite seleccionar uno.
 * Tambien permite indicar sii el trayecto se
 * realizara a pie o en coche.
 * 
 * @author Juan Millan
 *
 */
public class SelecSitios extends FragmentListActivity {
    private String[] SitiosNom;
    private List<Sitio> lstSitios;
    private GestorSitios gestorSitios;
    private Button btnSitiosCancelar;
    private RadioButton rbtnPie;
	private RadioButton rbtnCoche;
    private String clave;
    private String modo;
    private boolean esAlta;
    private boolean valido=true;
    private Sitio sitio=new Sitio();
    public static final String DATO_DEVUELTO1="clave";
    public static final String DATO_DEVUELTO2="modo";

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.selec_sitio);
		
		try{
	        gestorSitios = new GestorSitios(this,GestorSitios.DB_NAME,null,GestorSitios.DB_VERSION);
	        gestorSitios.open();
	        
        }
        catch (Exception e){
        	showDialog(e.getMessage());
        	return;
        }
		
		btnSitiosCancelar = (Button) this.findViewById(R.id.btnSitiosCancelar);
		rbtnPie = (RadioButton)findViewById(R.id.radioPie);
		rbtnCoche = (RadioButton)findViewById(R.id.radioCoche);
		
		iniPantalla();
		
	}
	
	@Override
    public void onListItemClick(ListView l, View v, int position, long id) {
        super.onListItemClick(l, v, position, id);
        
        clave=SitiosNom[position];
        if (rbtnPie.isChecked()) {
        	modo="Pie";
        }
        if (rbtnCoche.isChecked()) {
        	modo="Coche";
        }
        devolverDato();
    }
	
	@Override
    public void onStart() {
        super.onStart();
        
        btnSitiosCancelar.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View arg0) {
            	terminar();
            }
            
        });

	}

	private void iniPantalla() {
		cargarLista();
		
		// adapter a la Lista del Fragment
		if (SitiosNom!=null){
	        setListAdapter(new ArrayAdapter<String>(this,
	                android.R.layout.simple_list_item_1, SitiosNom));
		}
		
	}
	
	private void terminar() {
		gestorSitios.close();
    	finish();
	}
		
	private void cargarLista() {
		try {
			SitiosNom=new String[0];
			lstSitios=gestorSitios.listSitios();
			int i=0;
			
			if (!lstSitios.isEmpty()) {
				SitiosNom=new String[lstSitios.size()];
				Iterator<Sitio> sitioIterator = lstSitios.iterator();
		        while (sitioIterator.hasNext()) {
		            Sitio sitio = (Sitio)sitioIterator.next();
		            SitiosNom[i]=sitio.getNombre();
		            if (i==0) {
		            	clave=SitiosNom[i];
		            }
		            i++;
		        }
		        esAlta=false;
			} else {
				esAlta=true;
			}
		} catch (ExceptionApp e) {
			showDialog(e.getMessage());
		}
				
	}
	
	private void devolverDato()
    {
        Intent dato = new Intent();
        dato.putExtra(DATO_DEVUELTO1, clave);
        dato.putExtra(DATO_DEVUELTO2, modo);
        setResult(android.app.Activity.RESULT_OK,dato );
        terminar();
    }
	
	/**
	 * muestra una alerta.
	 * 
	 * @param texto
	 */
	private void showDialog(String texto){
		DialogFragment df = new DialogBoxAlert(texto);
	    df.show(getSupportFragmentManager(), getResources().getString(R.string.dialogo_titulo));
	}
	
}
