package es.jmillandel.senassistant.bbdd;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
 
import android.content.Context;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteDatabase.CursorFactory;
import android.database.sqlite.SQLiteException;
import android.database.sqlite.SQLiteOpenHelper;
import android.util.Log;

import es.jmillandel.senassistant.app.R;

/**
 * Clase que define la BBDD.
 * 
 * @author Juan Millan
 *
 */
public class GestorBBDD extends SQLiteOpenHelper {
	private static String DB_PATH = "";
    public static String DB_NAME = "senass_db";
    public static final int DB_VERSION = 1;
    protected SQLiteDatabase myDataBase;
    protected final Context myContext;
    
    private static final String CREATE_TABLE_MENSAJE = "CREATE TABLE Mensaje (idMensaje INTEGER PRIMARY KEY AUTOINCREMENT,titulo TEXT,texto TEXT,situacion INTEGER);";
    private static final String CREATE_INDEX_MENSAJE = "CREATE INDEX Mensaje_indexTitulo on Mensaje(titulo);";
    private static final String CREATE_TABLE_SITIO = "CREATE TABLE Sitio (idSitio INTEGER PRIMARY KEY AUTOINCREMENT,nombre TEXT,direccion TEXT,longitud REAL,latitud REAL,coslat REAL,sinlat REAL,coslng REAL,sinlng REAL);";
    private static final String CREATE_INDEX_SITIO = "CREATE INDEX Sitio_indexNombre on Sitio(nombre);";
    private static final String CREATE_TABLE_TRAYECTO = "CREATE TABLE Trayecto (idTrayecto INTEGER PRIMARY KEY AUTOINCREMENT,diaHora TEXT,longitud REAL,latitud REAL,direccion TEXT,tiempoPermanece REAL,tiempoTranscurrido REAL,distancia REAL);";
    private static final String CREATE_INDEX_TRAYECTO = "CREATE INDEX Trayecto_indexDia on Trayecto(diaHora);";
    private static final String CREATE_TABLE_DATOSGENERALES = "CREATE TABLE DatosGenerales (nombre TEXT PRIMARY KEY,telefono TEXT,contacto TEXT);";
    private static final String CREATE_TABLE_DESTINATARIO = "CREATE TABLE Destinatario (idDestinatario INTEGER PRIMARY KEY AUTOINCREMENT,nombre TEXT,telefono TEXT);";
    private static final String CREATE_INDEX_DESTINATARIO = "CREATE INDEX Destinatario_indexNombre on Destinatario(nombre);";
    private static final String CREATE_TABLE_AVISOS = "CREATE TABLE Aviso (sitio INTEGER,mensaje INTEGER,destinatario INTEGER,condicion INTEGER,tiempo REAL,PRIMARY KEY(Sitio, mensaje, destinatario, condicion),FOREIGN KEY(sitio) REFERENCES Sitio(idSitio),FOREIGN KEY(mensaje) REFERENCES Mensaje(idMensaje),FOREIGN KEY(destinatario)REFERENCES Destinatario(idDestinatario));";
 
    public GestorBBDD(Context contexto, String nombre, CursorFactory factory,
            int version) {
 
        super(contexto, nombre, factory, version);
        
        if(android.os.Build.VERSION.SDK_INT >= 4.2){
            DB_PATH = contexto.getApplicationInfo().dataDir + "/databases/";         
        } else {
           DB_PATH = "/data/data/" + contexto.getPackageName() + "/databases/";
        }
        this.myContext = contexto;

    }
 
    @Override
    public void onCreate(SQLiteDatabase db) {
    	try {
            createSenAssDB(db);
            
        } catch (IOException e) {
            throw new Error(myContext.getString(R.string.error_creando_bbdd));
        }
    	
    }
 
    @Override
    public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
        // Cuando haya cambios en la estructura
    }   
     
    /**
    * abre la bd.
    *  
    * @throws SQLException
    */
    public void open() throws SQLException {
    	 
    	myDataBase = this.getWritableDatabase();
    }

    /**
     * cierra la bd.
     * 
     */
    @Override
    public synchronized void close() {
 
        if (myDataBase != null)
            myDataBase.close();
 
        super.close();
    }
 
    /**
     * crea las tablas de la bd.
     * 
     * @param db base de datos
     * @throws IOException
     */
    private void createSenAssDB(SQLiteDatabase db) throws IOException {
    	db.execSQL(CREATE_TABLE_MENSAJE);
    	db.execSQL(CREATE_INDEX_MENSAJE);
    	db.execSQL(CREATE_TABLE_SITIO);
    	db.execSQL(CREATE_INDEX_SITIO);
    	db.execSQL(CREATE_TABLE_TRAYECTO);
    	db.execSQL(CREATE_INDEX_TRAYECTO);
    	db.execSQL(CREATE_TABLE_DATOSGENERALES);
    	db.execSQL(CREATE_TABLE_DESTINATARIO);
    	db.execSQL(CREATE_INDEX_DESTINATARIO);
    	db.execSQL(CREATE_TABLE_AVISOS);
    }
 
}