package es.jmillandel.senassistant.bbdd;

import android.content.Context;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteDatabase.CursorFactory;
import android.content.ContentValues;
import android.database.Cursor;
import java.util.ArrayList;
import java.util.List;


import es.jmillandel.senassistant.app.R;
import es.jmillandel.senassistant.beans.Aviso;
import es.jmillandel.senassistant.beans.Destinatario;
import es.jmillandel.senassistant.beans.ExceptionApp;
import es.jmillandel.senassistant.beans.Mensaje;

/**
 * Clase para gestionar los destinatarios en la BD.
 * Hereda de GestorBBDD.
 * 
 * @author Juan Millan
 *
 */
public class GestorDestinatarios extends GestorBBDD {

	public GestorDestinatarios(Context contexto, String nombre, CursorFactory factory,
            int version) {
		super(contexto, nombre, factory, version);
	}
	
	/**
	 * Alta destinatario
	 * 
	 * @param destinatario
	 * @throws ExceptionApp
	 */
	public void altaDestinatario(Destinatario destinatario) throws ExceptionApp {
		ContentValues nuevo = new ContentValues();
		nuevo.put("nombre", destinatario.getNombre());
		nuevo.put("telefono", destinatario.getTelefono());
		try {
			myDataBase.insert("Destinatario", null, nuevo);
		} catch (SQLException e) {
			throw new ExceptionApp(myContext.getString(R.string.error_destina_alta));
		}
		
	}
	
	/**
	 * Devuelve un destinatario buscado por su identificador
	 *
	 * @param idDestinatario
	 * @return Destinatario
	 * @throws ExceptionApp
	 */
	public Destinatario getDestinatario(int idDestinatario) throws ExceptionApp {
		return getDestinatarioWhere("idDestinatario="+idDestinatario);
	}
	
	/**
	 * Devuelve un destinatario buscado por su nombre
	 * 
	 * @param nombre
	 * @return Destinatario
	 * @throws ExceptionApp
	 */
	public Destinatario getDestinatario(String nombre) throws ExceptionApp {
		return getDestinatarioWhere("nombre='"+nombre+"'");
	}
	
	/**
	 * Devuelve un destinatario buscado por una condicion, si condicion vacia devuelve el primero 
	 * 
	 * @param where
	 * @return destinatario
	 * @throws ExceptionApp
	 */
	public Destinatario getDestinatarioWhere(String where) throws ExceptionApp {
		Destinatario destinatario=new Destinatario();
		String selectWhere="";
		
		if (where.length()>0) {
			selectWhere = " WHERE "+where;
		}
		
		try {
			Cursor c = myDataBase.rawQuery("SELECT * FROM Destinatario"+selectWhere, null);
			
	        if (c.moveToFirst()) {
	        	destinatario.setIdDestinatario(c.getInt(0));    
	        	destinatario.setNombre(c.getString(1));
	        	destinatario.setTelefono(c.getString(2));
	        	
	        	c.close();
	            return destinatario;
	        }
	        else
	        {
	        	c.close();
	        	return null;
	        }
		} catch (SQLException e) {
			throw new ExceptionApp(myContext.getString(R.string.error_destina_lee));
		}
	}
	
	/**
	 * Modifica un destinatario
	 * 
	 * @param destinatario
	 * @throws ExceptionApp
	 */
	public void modDestinatario(Destinatario destinatario) throws ExceptionApp {
		ContentValues nuevo = new ContentValues();
		nuevo.put("nombre", destinatario.getNombre());
		nuevo.put("telefono", destinatario.getTelefono());
		
		try {
			myDataBase.update("Destinatario", nuevo, "idDestinatario="+destinatario.getIdDestinatario(),null);
		} catch (SQLException e) {
			throw new ExceptionApp(myContext.getString(R.string.error_destina_mod));
		}		
	}
	
	/**
	 * Borra un destinatario
	 * 
	 * @param destinatario
	 * @throws ExceptionApp
	 */
	public void bajaDestinatario(Destinatario destinatario) throws ExceptionApp {
		
		try {
			myDataBase.delete("Destinatario", "idDestinatario="+destinatario.getIdDestinatario(),null);
		} catch (SQLException e) {
			throw new ExceptionApp(myContext.getString(R.string.error_destina_baja));
		}
		
	}
	
	/**
	 * Devuelve una lista con los destinatarios ordenada por nombre
	 * 
	 * @return List<Destinatario>
	 * @throws ExceptionApp
	 */
	public List<Destinatario> listDestinatarios() throws ExceptionApp {
		
		List<Destinatario> lstDestinatarios = new ArrayList<Destinatario>();     
	
		try {
	        Cursor c = myDataBase.rawQuery("SELECT * FROM Destinatario ORDER BY nombre", null);
	        if (c.moveToFirst()) { 
	            do {
	            	Destinatario destinatario = new Destinatario();
	            	destinatario.setIdDestinatario(c.getInt(0));    
	            	destinatario.setNombre(c.getString(1));
	            	destinatario.setTelefono(c.getString(2));       	 
		            lstDestinatarios.add(destinatario);
	            } while(c.moveToNext());
	        }
	        c.close();
	        return lstDestinatarios;     
		} catch (SQLException e) {
			throw new ExceptionApp(myContext.getString(R.string.error_destina_lista));
		}
	}
	
	/**
	 * devuelve un aviso de un destinatario.
	 * 
	 * @param destinatario
	 * @return Aviso
	 * @throws ExceptionApp
	 */
	public Aviso getAvisoDestinatario(int destinatario) throws ExceptionApp {
		Aviso aviso=new Aviso();
		
		try{
			GestorSitios gestorSitios= new GestorSitios(myContext,GestorSitios.DB_NAME,null,GestorSitios.DB_VERSION);
			gestorSitios.open();
	        aviso=gestorSitios.getAvisoDestinatario(destinatario);
	        gestorSitios.close();
	        return aviso;
        }
        catch (ExceptionApp ea){
        	throw new ExceptionApp(ea.getMessage());
        }
	}
	
}
