package es.jmillandel.senassistant.bbdd;

import android.content.Context;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteDatabase.CursorFactory;
import android.content.ContentValues;
import android.database.Cursor;

import es.jmillandel.senassistant.app.R;
import es.jmillandel.senassistant.beans.DatosGenerales;
import es.jmillandel.senassistant.beans.ExceptionApp;

/**
 * Clase para gestionar los datos generales en la BD.
 * Hereda de GestorBBDD.
 * 
 * @author Juan Millan
 *
 */
public class GestorGeneral extends GestorBBDD {

	public GestorGeneral(Context contexto, String nombre, CursorFactory factory,
            int version) {
		super(contexto, nombre, factory, version);
		
	}
	
	/**
	 * Alta datos generales
	 * 
	 * @param datGen
	 * @throws ExceptionApp
	 */
	public void altaDatosGen(DatosGenerales datGen) throws ExceptionApp {
		ContentValues nuevo = new ContentValues();
		nuevo.put("nombre", datGen.getNombre());
		nuevo.put("telefono", datGen.getTelefono());
		nuevo.put("contacto", datGen.getContacto());
		
		try {
			myDataBase.insert("DatosGenerales", null, nuevo);
		} catch (SQLException e) {
			throw new ExceptionApp(myContext.getString(R.string.error_datgen_alta));
		}
		
	}
	
	/**
	 * Devuelve el registro con los datos generales
	 * 
	 * @return DatosGenerales
	 * @throws ExceptionApp
	 */
	public DatosGenerales getDatosGen() throws ExceptionApp {
		DatosGenerales datGen=new DatosGenerales();
		
		try {
			Cursor c = myDataBase.rawQuery("SELECT * FROM DatosGenerales", null);
			
	        if (c.moveToFirst()) { 
	        	 datGen.setNombre(c.getString(0));    
	        	 datGen.setTelefono(c.getString(1));
	        	 datGen.setContacto(c.getString(2));
	        	 c.close();
	             return datGen;
	            }
	        else
	        {
	        	c.close();
	        	return null;
	        }
		} catch (SQLException e) {
			throw new ExceptionApp(myContext.getString(R.string.error_datgen_lee));
		}
	}
	
	/**
	 * Modifica el registro de los datos generales
	 * 
	 * @param datGen
	 * @param clave
	 * @throws ExceptionApp
	 */
	public void modDatosGen(DatosGenerales datGen, String clave) throws ExceptionApp {
		ContentValues nuevo = new ContentValues();
		nuevo.put("nombre", datGen.getNombre());
		nuevo.put("telefono", datGen.getTelefono());
		nuevo.put("contacto", datGen.getContacto());
		
		try {
			myDataBase.update("DatosGenerales", nuevo, "nombre='"+clave+"'",null);
		} catch (SQLException e) {
			throw new ExceptionApp(myContext.getString(R.string.error_datgen_mod));
		}
		
	}

}
