package es.jmillandel.senassistant.bbdd;

import android.content.Context;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteDatabase.CursorFactory;
import android.content.ContentValues;
import android.database.Cursor;
import java.util.ArrayList;
import java.util.List;

import es.jmillandel.senassistant.app.R;
import es.jmillandel.senassistant.beans.Aviso;
import es.jmillandel.senassistant.beans.ExceptionApp;
import es.jmillandel.senassistant.beans.Mensaje;

/**
 * Clase para gestionar los datos de los mensajes en la BD.
 * Hereda de GestorBBDD.
 * 
 * @author Juan Millan
 *
 */
public class GestorMensajes extends GestorBBDD {

	public GestorMensajes(Context contexto, String nombre, CursorFactory factory,
            int version) {
		super(contexto, nombre, factory, version);
	}
	
	/**
	 * Alta mensaje
	 * 
	 * @param mensaje
	 * @throws ExceptionApp
	 */
	public void altaMensaje(Mensaje mensaje) throws ExceptionApp {
		ContentValues nuevo = new ContentValues();
		nuevo.put("titulo", mensaje.getTitulo());
		nuevo.put("texto", mensaje.getTexto());
		if (mensaje.isSituacion()) {
			nuevo.put("situacion", 1);
		} else {
			nuevo.put("situacion", 0);
		}
		
		try {
			myDataBase.insert("Mensaje", null, nuevo);
		} catch (SQLException e) {
			throw new ExceptionApp(myContext.getString(R.string.error_mensaje_alta));
		}
	}
	
	/**
	 * Devuelve un mensaje buscado por su identificador
	 *
	 * @param idMensaje
	 * @return Mensaje
	 * @throws ExceptionApp
	 */
	public Mensaje getMensaje(int idMensaje) throws ExceptionApp {
		return getMensajeWhere("idMensaje="+idMensaje);
	}
	
	/**
	 * Devuelve un mensaje buscado por su titulo
	 * 
	 * @param titulo
	 * @return Mensaje
	 * @throws ExceptionApp
	 */
	public Mensaje getMensaje(String titulo) throws ExceptionApp {
		return getMensajeWhere("titulo='"+titulo+"'");
	}
	
	/**
	 * Devuelve un mensaje buscado por una condicion, si condicion vacia devuelve el primero 
	 * 
	 * @param where
	 * @return mensaje
	 * @throws ExceptionApp
	 */
	public Mensaje getMensajeWhere(String where) throws ExceptionApp {
		Mensaje mensaje=new Mensaje();
		String selectWhere="";
		
		if (where.length()>0) {
			selectWhere = " WHERE "+where;
		}
		
		try {
			Cursor c = myDataBase.rawQuery("SELECT * FROM Mensaje"+selectWhere, null);
			
	        if (c.moveToFirst()) {
	        	mensaje.setIdMensaje(c.getInt(0));    
	        	mensaje.setTitulo(c.getString(1));
	        	mensaje.setTexto(c.getString(2));
	        	if (c.getInt(3)==0) {
	        		mensaje.setSituacion(false);
	        	} else {
	        		mensaje.setSituacion(true);
	        	}
	        	 c.close();
	             return mensaje;
	            }
	        else
	        {
	        	c.close();
	        	return null;
	        }
		} catch (SQLException e) {
			throw new ExceptionApp(myContext.getString(R.string.error_mensaje_lee));
		}
	}
	
	/**
	 * Modifica un mensaje
	 * 
	 * @param mensaje
	 * @throws ExceptionApp
	 */
	public void modMensaje(Mensaje mensaje) throws ExceptionApp {
		ContentValues nuevo = new ContentValues();
		nuevo.put("titulo", mensaje.getTitulo());
		nuevo.put("texto", mensaje.getTexto());
		if (mensaje.isSituacion()) {
			nuevo.put("situacion", 1);
		} else {
			nuevo.put("situacion", 0);
		}
		
		try {
			myDataBase.update("Mensaje", nuevo, "idMensaje="+mensaje.getIdMensaje(),null);
		} catch (SQLException e) {
			throw new ExceptionApp(myContext.getString(R.string.error_mensaje_mod));
		}
	}
	
	/**
	 * Borra un mensaje
	 * 
	 * @param mensaje
	 * @throws ExceptionApp
	 */
	public void bajaMensaje(Mensaje mensaje) throws ExceptionApp {
		
		try {
			myDataBase.delete("Mensaje", "idMensaje="+mensaje.getIdMensaje(),null);
		} catch (SQLException e) {
			throw new ExceptionApp(myContext.getString(R.string.error_mensaje_baja));
		}
	}
	
	/**
	 * Devuelve una lista con los mensajes ordenada por titulo
	 * 
	 * @return List<Mensaje>
	 * @throws ExceptionApp
	 */
	public List<Mensaje> listMensajes() throws ExceptionApp {
		
		List<Mensaje> lstMensajes = new ArrayList<Mensaje>();
		
		try {
	        Cursor c = myDataBase.rawQuery("SELECT * FROM Mensaje ORDER BY titulo", null);
	        if (c.moveToFirst()) { 
	            do {
	            	Mensaje mensaje = new Mensaje();
	            	mensaje.setIdMensaje(c.getInt(0));    
	            	mensaje.setTitulo(c.getString(1));
	            	mensaje.setTexto(c.getString(2));
	            	if (c.getInt(3)==0) {
	            		mensaje.setSituacion(false);
	            	} else {
	            		mensaje.setSituacion(true);
	            	}       	 
		            lstMensajes.add(mensaje);
	            } while(c.moveToNext());
	        }
	        c.close();
	        return lstMensajes;
		} catch (SQLException e) {
			throw new ExceptionApp(myContext.getString(R.string.error_mensaje_lista));
		}
	}
	
	/**
	 * devuelve un aviso buscado por el mensaje.
	 * 
	 * @param mensaje
	 * @return Aviso
	 * @throws ExceptionApp
	 */
	public Aviso getAvisoMensaje(int mensaje) throws ExceptionApp {
		Aviso aviso=new Aviso();
		
		try{
			GestorSitios gestorSitios= new GestorSitios(myContext,GestorSitios.DB_NAME,null,GestorSitios.DB_VERSION);
			gestorSitios.open();
	        aviso=gestorSitios.getAvisoMensaje(mensaje);
	        gestorSitios.close();
	        return aviso;
        }
        catch (ExceptionApp ea){
        	throw new ExceptionApp(ea.getMessage());
        }
	}
	
}
