package es.jmillandel.senassistant.bbdd;

import android.content.Context;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteDatabase.CursorFactory;
import android.content.ContentValues;
import android.database.Cursor;
import android.util.Log;

import java.util.ArrayList;
import java.util.List;

import es.jmillandel.senassistant.app.R;
import es.jmillandel.senassistant.beans.Aviso;
import es.jmillandel.senassistant.beans.Condicion;
import es.jmillandel.senassistant.beans.Destinatario;
import es.jmillandel.senassistant.beans.ExceptionApp;
import es.jmillandel.senassistant.beans.Mensaje;
import es.jmillandel.senassistant.beans.Sitio;

/**
 * Clase para gestionar los datos de los sitios y avisos en la BD.
 * Hereda de GestorBBDD.
 * 
 * @author Juan Millan
 *
 */
public class GestorSitios extends GestorBBDD {

	public GestorSitios(Context contexto, String nombre, CursorFactory factory,
            int version) {
		super(contexto, nombre, factory, version);
	}
	
	/**
	 * Alta sitio
	 * 
	 * @param sitio
	 * @throws ExceptionApp
	 */
	public void altaSitio(Sitio sitio) throws ExceptionApp {
		ContentValues nuevo = new ContentValues();
		nuevo.put("nombre", sitio.getNombre());
		nuevo.put("direccion", sitio.getDireccion());
		nuevo.put("longitud", sitio.getLongitud());
		nuevo.put("latitud", sitio.getLatitud());
		nuevo.put("coslat", sitio.getCosLat());
		nuevo.put("sinlat", sitio.getSinLat());
		nuevo.put("coslng", sitio.getCosLng());
		nuevo.put("sinlng", sitio.getSinLng());
		
		try {
			myDataBase.insert("Sitio", null, nuevo);
		} catch (SQLException e) {
			throw new ExceptionApp(myContext.getString(R.string.error_sitio_alta)); 
		}
	}
	
	/**
	 * Devuelve un sitio buscado por su identificador
	 *
	 * @param idSitio
	 * @return Sitio
	 * @throws ExceptionApp
	 */
	public Sitio getSitio(int idSitio) throws ExceptionApp {
		return getSitioWhere("idSitio="+idSitio);
	}
	
	/**
	 * Devuelve un sitio buscado por su nombre
	 * 
	 * @param nombre
	 * @return Sitio
	 * @throws ExceptionApp
	 */
	public Sitio getSitio(String nombre) throws ExceptionApp {
		return getSitioWhere("nombre='"+nombre+"'");
	}
	
	/**
	 * Devuelve un sitio buscado por su longitud y latitud
	 * 
	 * @param longitud
	 * @param latitud
	 * @return Sitio
	 * @throws ExceptionApp
	 */
	public Sitio getSitio(double longitud, double latitud) throws ExceptionApp {
		return getSitioWhere("longitud="+longitud+" and latitud="+latitud);
	}
	
	/**
	 * Devuelve un sitio buscado por una condicion, si condicion vacia devuelve el primero 
	 * 
	 * @return where
	 * @throws ExceptionApp
	 */
	public Sitio getSitioWhere(String where) throws ExceptionApp {
		Sitio sitio=new Sitio();
		String selectWhere="";
		
		if (where.length()>0) {
			selectWhere = " WHERE "+where;
		}
		
		try {
			Cursor c = myDataBase.rawQuery("SELECT * FROM Sitio"+selectWhere, null);
			
	        if (c.moveToFirst()) {
	        	 sitio.setIdSitio(c.getInt(0));    
	        	 sitio.setNombre(c.getString(1));
	        	 sitio.setDireccion(c.getString(2));
	        	 sitio.setLongitud(c.getDouble(3));
	        	 sitio.setLatitud(c.getDouble(4));
	        	 c.close();
	             return sitio;
	            }
	        else
	        {
	        	c.close();
	        	return null;
	        }
		} catch (SQLException e) {
			throw new ExceptionApp(myContext.getString(R.string.error_sitio_lee));
		}
	}
	
	/**
	 * Modifica un sitio
	 * 
	 * @param sitio
	 * @throws ExceptionApp
	 */
	public void modSitio(Sitio sitio) throws ExceptionApp {
		ContentValues nuevo = new ContentValues();
		nuevo.put("nombre", sitio.getNombre());
		nuevo.put("direccion", sitio.getDireccion());
		nuevo.put("longitud", sitio.getLongitud());
		nuevo.put("latitud", sitio.getLatitud());
		nuevo.put("coslat", sitio.getCosLat());
		nuevo.put("sinlat", sitio.getSinLat());
		nuevo.put("coslng", sitio.getCosLng());
		nuevo.put("sinlng", sitio.getSinLng());
		
		try {
			myDataBase.update("Sitio", nuevo, "idSitio="+sitio.getIdSitio(),null);
		} catch (SQLException e) {
			throw new ExceptionApp(myContext.getString(R.string.error_sitio_mod));
		}
	}
	
	/**
	 * Borra un sitio, primero borra sus avisos en caso de que tenga.
	 * 
	 * @param sitio
	 * @throws ExceptionApp
	 */
	public void bajaSitio(Sitio sitio) throws ExceptionApp {
		try {
			myDataBase.delete("Aviso", "sitio="+sitio.getIdSitio(),null);
			myDataBase.delete("Sitio", "idSitio="+sitio.getIdSitio(),null);
		} catch (SQLException e) {
			throw new ExceptionApp(myContext.getString(R.string.error_sitio_baja));
		}
	}
	
	/**
	 * Devuelve una lista con los sitios ordenada por nombre
	 * 
	 * @return List<Sitio>
	 * @throws ExceptionApp
	 */
	public List<Sitio> listSitios() throws ExceptionApp {
		
		List<Sitio> lstSitios = new ArrayList<Sitio>();     
		
		try {
	        Cursor c = myDataBase.rawQuery("SELECT * FROM Sitio ORDER BY nombre", null);
	        if (c.moveToFirst()) { 
	            do {
		            Sitio sitio = new Sitio();
		            sitio.setIdSitio(c.getInt(0));    
		        	sitio.setNombre(c.getString(1));
		        	sitio.setDireccion(c.getString(2));
		        	sitio.setLongitud(c.getDouble(3));
		        	sitio.setLatitud(c.getDouble(4));       	 
		            lstSitios.add(sitio);
	            } while(c.moveToNext());
	        }
	        c.close();
	        return lstSitios;
		} catch (SQLException e) {
			throw new ExceptionApp(myContext.getString(R.string.error_sitio_lista));
		}
	}
	
	/**
	 * Alta Aviso
	 * 
	 * @param aviso
	 * @throws ExceptionApp
	 */
	public void altaAviso(Aviso aviso) throws ExceptionApp {
		ContentValues nuevo = new ContentValues();
		nuevo.put("sitio", aviso.getSitio().getIdSitio());
		nuevo.put("mensaje", aviso.getMensaje().getIdMensaje());
		nuevo.put("destinatario", aviso.getDestinatario().getIdDestinatario());
		nuevo.put("condicion", aviso.getCondicion().ordinal());
		nuevo.put("tiempo", aviso.getTiempo());
		
		try {
			myDataBase.insert("Aviso", null, nuevo);
		} catch (SQLException e) {
			throw new ExceptionApp(myContext.getString(R.string.error_aviso_alta));
		}
	}
	
	/**
	 * Modifica Aviso
	 * 
	 * @param aviso
	 * @param avisoOld
	 * @throws ExceptionApp
	 */
	public void modAviso(Aviso aviso, Aviso avisoOld) throws ExceptionApp {
		ContentValues nuevo = new ContentValues();
		String clave="sitio="+avisoOld.getSitio().getIdSitio()+" and mensaje="+avisoOld.getMensaje().getIdMensaje()+" and destinatario="+avisoOld.getDestinatario().getIdDestinatario()+" and condicion="+avisoOld.getCondicion().ordinal();
		
		nuevo.put("sitio", aviso.getSitio().getIdSitio());
		nuevo.put("mensaje", aviso.getMensaje().getIdMensaje());
		nuevo.put("destinatario", aviso.getDestinatario().getIdDestinatario());
		nuevo.put("condicion", aviso.getCondicion().ordinal());
		nuevo.put("tiempo", aviso.getTiempo());
		
		try {
			myDataBase.update("Aviso", nuevo, clave,null);
		} catch (SQLException e) {
			throw new ExceptionApp(myContext.getString(R.string.error_aviso_mod));
		}
	}
	
	/**
	 * Borra un Aviso
	 * 
	 * @param aviso
	 * @throws ExceptionApp
	 */
	public void bajaAviso(Aviso aviso) throws ExceptionApp {
		String clave="sitio="+aviso.getSitio().getIdSitio()+" and mensaje="+aviso.getMensaje().getIdMensaje()+" and destinatario="+aviso.getDestinatario().getIdDestinatario()+" and condicion="+aviso.getCondicion().ordinal();
		
		try {
			myDataBase.delete("Aviso", clave,null);
		} catch (SQLException e) {
			throw new ExceptionApp(myContext.getString(R.string.error_aviso_baja));
		}
	}
	
	/**
	 * devuelve una lista con los avisos de un sitio.
	 * 
	 * @param idSitio
	 * @return List<Aviso> lista con los avisos
	 * @throws ExceptionApp
	 */
	public List<Aviso> listAvisos(int idSitio) throws ExceptionApp {
		
		List<Aviso> lstAvisos = new ArrayList<Aviso>();     
	
		try {
	        Cursor c = myDataBase.rawQuery("SELECT * FROM Aviso WHERE sitio="+idSitio+" ", null);
	        if (c.moveToFirst()) { 
	            do {
		            Aviso aviso = new Aviso();
		            aviso.setSitio(getSitio(c.getInt(0)));    
		        	aviso.setMensaje(getMensaje(c.getInt(1)));
		        	aviso.setDestinatario(getDestinatario(c.getInt(2)));
		        	aviso.setCondicion(Condicion.values()[c.getInt(3)]);
		        	aviso.setTiempo(c.getLong(4));    	 
		            lstAvisos.add(aviso);
	            } while(c.moveToNext());
	        }
	        c.close();
	        return lstAvisos;
		} catch (SQLException e) {
			throw new ExceptionApp(myContext.getString(R.string.error_aviso_lista));
		}
	}
	
	/**
	 * devuelve un aviso por su clave principal.
	 * 
	 * @param sitio
	 * @param mensaje
	 * @param destinatario
	 * @param condicion
	 * @return Aviso
	 * @throws ExceptionApp
	 */
	public Aviso getAviso(int sitio, int mensaje, int destinatario, int condicion) throws ExceptionApp {
		Aviso aviso=new Aviso();
		String clave="sitio="+sitio+" and mensaje="+mensaje+" and destinatario="+destinatario+" and condicion="+condicion;
		
		try {
			Cursor c = myDataBase.rawQuery("SELECT * FROM Aviso WHERE "+clave, null);
			
		    if (c.moveToFirst()) {
		    	aviso.setSitio(getSitio(c.getInt(0)));    
	        	aviso.setMensaje(getMensaje(c.getInt(1)));
	        	aviso.setDestinatario(getDestinatario(c.getInt(2)));
	        	aviso.setCondicion(Condicion.values()[c.getInt(3)]);
	        	aviso.setTiempo(c.getLong(4));
		    	 c.close();
		         return aviso;
		        }
		    else
		    {
		    	c.close();
		    	return null;
		    }
		} catch (SQLException e) {
			throw new ExceptionApp(myContext.getString(R.string.error_aviso_lee));
		} catch (ExceptionApp ea) {
			throw new ExceptionApp(ea.getMessage());
		} 
	}
	
	/**
	 * devuelve el primer aviso por el mensaje.
	 * 
	 * @param mensaje
	 * @return Aviso
	 * @throws ExceptionApp
	 */
	public Aviso getAvisoMensaje(int mensaje) throws ExceptionApp {
		Aviso aviso=new Aviso();
		String clave="mensaje="+mensaje;
		
		try {
			Cursor c = myDataBase.rawQuery("SELECT * FROM Aviso WHERE "+clave, null);
			
		    if (c.moveToFirst()) {
		    	aviso.setSitio(getSitio(c.getInt(0)));    
	        	aviso.setMensaje(getMensaje(c.getInt(1)));
	        	aviso.setDestinatario(getDestinatario(c.getInt(2)));
	        	aviso.setCondicion(Condicion.values()[c.getInt(3)]);
	        	aviso.setTiempo(c.getLong(4));
		    	 c.close();
		         return aviso;
		        }
		    else
		    {
		    	c.close();
		    	return null;
		    }
		} catch (SQLException e) {
			throw new ExceptionApp(myContext.getString(R.string.error_aviso_lee));
		} catch (ExceptionApp ea) {
			throw new ExceptionApp(ea.getMessage());
		} 
	}
	
	/**
	 * devuelve el primer aviso por el destinatario.
	 * 
	 * @param destinatario
	 * @return Aviso
	 * @throws ExceptionApp
	 */
	public Aviso getAvisoDestinatario(int destinatario) throws ExceptionApp {
		Aviso aviso=new Aviso();
		String clave="destinatario="+destinatario;
		
		try {
			Cursor c = myDataBase.rawQuery("SELECT * FROM Aviso WHERE "+clave, null);
			
		    if (c.moveToFirst()) {
		    	aviso.setSitio(getSitio(c.getInt(0)));    
	        	aviso.setMensaje(getMensaje(c.getInt(1)));
	        	aviso.setDestinatario(getDestinatario(c.getInt(2)));
	        	aviso.setCondicion(Condicion.values()[c.getInt(3)]);
	        	aviso.setTiempo(c.getLong(4));
		    	 c.close();
		         return aviso;
		        }
		    else
		    {
		    	c.close();
		    	return null;
		    }
		} catch (SQLException e) {
			throw new ExceptionApp(myContext.getString(R.string.error_aviso_lee));
		} catch (ExceptionApp ea) {
			throw new ExceptionApp(ea.getMessage());
		} 
	}
	
	/**
	 * Devuelve un mensaje por su clave
	 * 
	 * @param idMensaje
	 * @return Mensaje
	 * @throws ExceptionApp
	 */
	public Mensaje getMensaje(int idMensaje) throws ExceptionApp {
		Mensaje mensaje=new Mensaje();
		
		try{
			GestorMensajes gestorMensajes= new GestorMensajes(myContext,GestorMensajes.DB_NAME,null,GestorMensajes.DB_VERSION);
	        gestorMensajes.open();
	        mensaje=gestorMensajes.getMensaje(idMensaje);
	        gestorMensajes.close();
	        return mensaje;
        }
        catch (ExceptionApp ea){
        	throw new ExceptionApp(ea.getMessage());
        }	
	}
	
	/**
	 * Devuelve un mensaje por su titulo
	 * 
	 * @param titulo
	 * @return Mensaje
	 * @throws ExceptionApp
	 */
	public Mensaje getMensaje(String titulo) throws ExceptionApp {
		Mensaje mensaje=new Mensaje();
		
		try{
			GestorMensajes gestorMensajes= new GestorMensajes(myContext,GestorMensajes.DB_NAME,null,GestorMensajes.DB_VERSION);
	        gestorMensajes.open();
	        mensaje=gestorMensajes.getMensaje(titulo);
	        gestorMensajes.close();
	        return mensaje;
        }
        catch (ExceptionApp ea){
        	throw new ExceptionApp(ea.getMessage());
        }	
	}
	
	/**
	 * Devuelve un destinatario por su clave
	 * 
	 * @param idDestinatario
	 * @return Destinatario
	 * @throws ExceptionApp
	 */
	public Destinatario getDestinatario(int idDestinatario) throws ExceptionApp {
		Destinatario destina=new Destinatario();
		
		try{
			GestorDestinatarios gestorDestinatarios= new GestorDestinatarios(myContext,GestorDestinatarios.DB_NAME,null,GestorDestinatarios.DB_VERSION);
			gestorDestinatarios.open();
	        destina=gestorDestinatarios.getDestinatario(idDestinatario);
	        gestorDestinatarios.close();
	        return destina;
        }
        catch (ExceptionApp ea){
        	throw new ExceptionApp(ea.getMessage());
        }	
	}
	
	/**
	 * Devuelve un destinatario por su nombre
	 * 
	 * @param nombre
	 * @return Destinatario
	 * @throws ExceptionApp
	 */
	public Destinatario getDestinatario(String nombre) throws ExceptionApp {
		Destinatario destina=new Destinatario();
		
		try{
			GestorDestinatarios gestorDestinatarios= new GestorDestinatarios(myContext,GestorDestinatarios.DB_NAME,null,GestorDestinatarios.DB_VERSION);
			gestorDestinatarios.open();
	        destina=gestorDestinatarios.getDestinatario(nombre);
	        gestorDestinatarios.close();
	        return destina;
        }
        catch (ExceptionApp ea){
        	throw new ExceptionApp(ea.getMessage());
        }	
	}

}
