package es.jmillandel.senassistant.bbdd;

import android.content.Context;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteDatabase.CursorFactory;
import android.content.ContentValues;
import android.database.Cursor;
import android.location.Location;
import android.telephony.SmsManager;
import android.util.Log;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Date;
import java.util.List;

import es.jmillandel.senassistant.app.R;
import es.jmillandel.senassistant.beans.ExceptionApp;
import es.jmillandel.senassistant.beans.Trayecto;

/**
 * Clase para gestionar el acceso a trayectos.
 * Hereda de GestorBBDD.
 * 
 * @author Juan Millan
 *
 */
public class GestorTrayecto extends GestorBBDD {

	public GestorTrayecto(Context contexto, String nombre, CursorFactory factory,
            int version) {
		super(contexto, nombre, factory, version);
	}
	
	/**
	 * devuelve la lista de los trayectos realizados en el dia.
	 * 
	 * @return List<Trayecto> lista de trayectos
	 * @throws ExceptionApp
	 */
	public List<Trayecto> listTrayecto() throws ExceptionApp {
		String dia;
		String diaIni;
		String diaFin;
		
		dia=new SimpleDateFormat("yyyy-MM-dd").format(new Date().getTime());
		diaIni=dia+" 00:00:00";
		diaFin=dia+" 23:59:59";
		List<Trayecto> lstTrayecto = new ArrayList<Trayecto>();
		
		String sqlText="SELECT * FROM Trayecto WHERE diaHora BETWEEN '"+diaIni+"' AND '"+diaFin+"' ORDER BY diaHora";

		try {
	        Cursor c = myDataBase.rawQuery(sqlText, null);
	        if (c.moveToFirst()) { 
	            do {
		            Trayecto trayecto = new Trayecto();
		            trayecto.setIdTrayecto(c.getInt(0));
		            try {
		            	trayecto.setDiaHora(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").parse(c.getString(1)));
		            } catch (ParseException Pe) {
		            	
		            }
		            trayecto.setLongitud(c.getDouble(2));
		            trayecto.setLatitud(c.getDouble(3));
		            trayecto.setDireccion(c.getString(4));	            
		        	trayecto.setTiempoPerm(c.getDouble(5));
		        	trayecto.setTiempoTrans(c.getDouble(6));
		        	trayecto.setDistancia(c.getDouble(7));
		            lstTrayecto.add(trayecto);
	            } while(c.moveToNext());
	        }
	        c.close();
		} catch (SQLException e) {
			throw new ExceptionApp(myContext.getString(R.string.error_trayecto_lista));
		}
        return lstTrayecto;     
	}
	
	
}
